/*
	Copyright (C) 2009-2012  Jesus Calvino-Fraga, jesusc (at) ece.ubc.ca
	   
	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2, or (at your option) any
	later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef __APPLE__
	#define __unix__ 1
#endif

#ifdef __unix__
	#include <limits.h>
	#ifdef __APPLE__
		#include <termios.h>
	#else
		#include <termio.h>
	#endif
	#include <stdio.h>
	#include <stdlib.h>
	#include <unistd.h>
	#include <fcntl.h>
	#include <sys/signal.h>
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <math.h>
	#include <time.h>
	#include <string.h>
	#include <errno.h>
#else
	#include <stdio.h>
	#include <stdlib.h>
	#include <conio.h>
	#include <windows.h>
	#include <time.h>
#endif

#define EQ(X,Y)  (_stricmp(X, Y)==0)
#define NEQ(X,Y) (_stricmp(X, Y)!=0)

#ifdef __unix__
	#define MAX_PATH PATH_MAX
	#define DWORD unsigned long int
	#define stricmp strcasecmp
	int fd;
	char SerialPort[MAX_PATH]="/dev/ttyUSB0";
	struct termios comio;
#else
	HANDLE hComm=INVALID_HANDLE_VALUE;
	int SerialPort=1;
#endif

char HexName[MAX_PATH]="";
char OutName[MAX_PATH]="";
char ProgName[MAX_PATH]="";
#define version "2.1"

unsigned char ReadFlash=0;
unsigned char VerifyFlash=0;
unsigned char Calibrate=0;
unsigned char RandomValues=0;
unsigned char SlowTx=0;

#define BUFFSIZE 0x100

unsigned char tx[BUFFSIZE], rx[BUFFSIZE];
unsigned char Atmel_Signature[BUFFSIZE];

int byte_offset=0;  // Byte offset.  It could be 0, 1, or 2 depending on the AT89LP type.

typedef struct
{
	  char name[0x20];
	  int size;
	  char ID_Str[7];
	  unsigned char buff_size; // For programming
} _at_part_ID_S2S;

const _at_part_ID_S2S at_part_ID_S2S[] = {
	{"AT89LP213",   0x800,   "1E27FF", 32 },
	{"AT89LP214",   0x800,   "1E28FF", 32 },
	{"AT89LP216",   0x800,   "1E29FF", 32 },
	{"AT89LP2052",  0x800,   "1E25FF", 32 },
	{"AT89LP4052",  0x1000,  "1E45FF", 32 },
	{"AT89LP428",   0x1000,  "1E40FF", 64 },
	{"AT89LP828",   0x2000,  "1E42FF", 64 },
	{"AT89LP51",    0x1000,  "1E5405", 64 },
	{"AT89LP52",    0x2000,  "1E5406", 64 },
	{"AT89LP3240",  0x8000,  "1E32FC", 64 },
	{"AT89LP6440",  0x10000, "1E64FF", 64 },
	{"AT89LP51RD2", 0x10000, "1E6472", 64 },
	{"AT89LP51ED2", 0x10000, "1E6465", 64 },
	{"AT89LP51ID2", 0x10000, "1E6469", 64 },
	{"NOT FOUND",   0x0000,  "000000", 0   } };

unsigned char FlashBuff[0x10000];
char Name_Str[BUFFSIZE];
int memsize;
char ID_Str[]="123456";
int buff_size;

// Fuses

int CS0=0;
int CS1=0;
int SUT0=1;
int SUT1=1;
int Reset=1;
int BODEnable=1;
int OCDDisable=1;
int ISP=1;
int UsrSigProDis=0;
int TristatePorts=0;
int OCDInt=0;
int IAPDisable=0;

#ifdef __unix__

/*****************************************************************************
kbhit() and getch() for Linux/UNIX
Chris Giese <geezer@execpc.com>	http://my.execpc.com/~geezer
Release date: ?
This code is public domain (no copyright).
You can do whatever you want with it.
*****************************************************************************/

static struct termios g_old_kbd_mode;

static void cooked(void)
{
	tcsetattr(0, TCSANOW, &g_old_kbd_mode);
}

static void raw(void)
{
	static char init;

	struct termios new_kbd_mode;

	if(init)
		return;
	/* put keyboard (stdin, actually) in raw, unbuffered mode */
	tcgetattr(0, &g_old_kbd_mode);
	memcpy(&new_kbd_mode, &g_old_kbd_mode, sizeof(struct termios));
	new_kbd_mode.c_lflag &= ~(ICANON | ECHO);
	new_kbd_mode.c_cc[VTIME] = 0;
	new_kbd_mode.c_cc[VMIN] = 1;
	tcsetattr(0, TCSANOW, &new_kbd_mode);
	/* when we exit, go back to normal, "cooked" mode */
	atexit(cooked);

	init = 1;
}

static int kbhit(void)
{
	struct timeval timeout;
	fd_set read_handles;
	int status;

	raw();
	/* check stdin (fd 0) for activity */
	FD_ZERO(&read_handles);
	FD_SET(0, &read_handles);
	timeout.tv_sec = timeout.tv_usec = 0;
	status = select(0 + 1, &read_handles, NULL, NULL, &timeout);
	if(status < 0)
	{
		printf("select() failed in kbhit()\n");
		exit(1);
	}
	return status;
}

static int getch(void)
{
	unsigned char temp;

	raw();
	/* stdin = fd 0 */
	if(read(0, &temp, 1) != 1)
		return 0;
	return temp;
}

#define NOPAIRTY 0
#define TWOSTOPBITS 0
#define NOPARITY 0

int Select_Baud (int Baud_Rate)
{
	switch (Baud_Rate)
	{
		case 115200: return B115200;
		case 57600:  return B57600;
		case 38400:  return B38400;
		case 19200:  return B19200;
		case 9600:   return B9600;
		case 4800:   return B4800;
		case 2400:   return B2400;
		case 1800:   return B1800;
		case 1200:   return B1200;
		default:     return B4800;
	}
}

int OpenSerialPort(char * devicename, int baud, int parity, int numbits, int numstop)
{
    struct termios options;
	speed_t BAUD;
	
	BAUD=Select_Baud(baud);
	
	//open the device(com port) to be non-blocking (read will return immediately)
	fd = open(devicename, O_RDWR | O_NOCTTY | O_NDELAY );
	if (fd < 0)
	{
		perror(devicename);
		return(1);
	}
	
	/*Reading data from a port is a little trickier. When you operate the port
	in raw data mode, each read system call will return however many characters
	are actually available in the serial input buffers. If no characters are
	available, the call will block (wait) until characters come in, an interval
	timer expires, or an error occurs. The read function can be made to return
	immediately by doing the following:*/
	
	fcntl(fd, F_SETFL, FNDELAY);

	/* The FNDELAY option causes the read function to return 0 if no characters
	are available on the port. To restore normal (blocking) behavior, call
	fcntl() without the FNDELAY option:*/
	
    //fcntl(fd, F_SETFL, 0);
	
	/*This is also used after opening a serial port with the O_NDELAY option.*/

	// Make the file descriptor asynchronous 
	
	tcgetattr(fd, &comio);
	
	//newtio.sg_ispeed = newtio.sg_ospeed = BAUD;
	cfsetospeed(&comio, (speed_t)BAUD);
	cfsetispeed(&comio, (speed_t)BAUD);
	
	comio.c_cflag = BAUD | CS8 | CSTOPB | CLOCAL | CREAD;
	comio.c_cflag &= ~(CRTSCTS); /* No hardware flow control */
	comio.c_iflag = IGNPAR;
	comio.c_iflag &= ~(IXON | IXOFF | IXANY); /* No software flow control */
	comio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); /* Raw input mode*/
	comio.c_oflag &= ~OPOST; /* Raw ouput mode */
	comio.c_cc[VMIN]=0;
	comio.c_cc[VTIME]=1;
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &comio);

	return(0);
}

void CloseSerialPort(void)
{
	close(fd);
}

#else

int OpenSerialPort (int Port, DWORD baud, BYTE parity, BYTE bits, BYTE stop)
{
	char sPort[16];
	DCB dcb;
	COMMTIMEOUTS Timeouts;

	sprintf(sPort, "\\\\.\\COM%d", Port);
	
	hComm = CreateFile(sPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hComm == INVALID_HANDLE_VALUE)
	{
		//printf("Failed to open up the comunication port COM%d\n", Port);
		return -1;
	}
	
	if (!GetCommState(hComm, &dcb))
	{
		printf("Failed in call to GetCommState()\n");
		return -1;
	}

	dcb.fAbortOnError=FALSE;
	dcb.BaudRate = baud; 
	dcb.Parity = parity;
	dcb.ByteSize = bits;
	dcb.StopBits = stop;
	dcb.fDsrSensitivity = FALSE;
	dcb.fOutxCtsFlow = FALSE;
	dcb.fOutxDsrFlow = FALSE;
	dcb.fOutX = FALSE;
	dcb.fInX = FALSE;

	//Now that we have all the settings in place, make the changes
	if (!SetCommState(hComm, &dcb))
	{
		printf("Failed in call to SetCommState()\n");
		return -1;
	}	

	ZeroMemory(&Timeouts, sizeof(COMMTIMEOUTS));
	Timeouts.ReadIntervalTimeout = 250;
	Timeouts.ReadTotalTimeoutMultiplier = 10;
	Timeouts.ReadTotalTimeoutConstant = 250;
	Timeouts.WriteTotalTimeoutMultiplier = 10;
	Timeouts.WriteTotalTimeoutConstant = 250;
	if (!SetCommTimeouts(hComm, &Timeouts))
	{
		printf("Failed in call to SetCommTimeouts\n");
		return -1;
	}

	return 0;
}

int CloseSerialPort (void)
{
	BOOL bSuccess;
	bSuccess = CloseHandle(hComm);
    hComm = INVALID_HANDLE_VALUE;
    if (!bSuccess)
    {
		printf("Failed to close serial port, GetLastError:%d\n", GetLastError());
		return -1;
	}
	return 0;
}

#endif

int hex2dec (char hex_digit)
{
   int j;
   j=toupper(hex_digit)-'0';
   if (j>9) j -= 7;
   return j;
}

unsigned char GetByte(char * buffer)
{
	return hex2dec(buffer[0])*0x10+hex2dec(buffer[1]);
}

unsigned short GetWord(char * buffer)
{
	return	hex2dec(buffer[0])*0x1000+
			hex2dec(buffer[1])*0x100+
			hex2dec(buffer[2])*0x10+
			hex2dec(buffer[3]);
}

long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}

int ReadHexFile(char * filename)
{
	char buffer[1024];
	FILE * filein;
	int j, numbytes;
	unsigned char linesize, recordtype, rchksum, value;
	unsigned short address;
	int MaxAddress=0;
	int chksum;
	int line_counter=0;
	int numread=0;

	//Set the buffer to its default value
	if(RandomValues==0)	memset(FlashBuff, 0xff, memsize);

    if ( (filein=fopen(filename, "r")) == NULL )
    {
       printf("Error: Can't open file `%s`.\r\n", filename);
       return -1;
    }
    
	numbytes=filesize(filein);

    while(fgets(buffer, sizeof(buffer), filein)!=NULL)
    {
    	numread+=(strlen(buffer)+1);

    	line_counter++;
    	if(buffer[0]==':')
    	{
			linesize = GetByte(&buffer[1]);
			address = GetWord(&buffer[3]);
			recordtype = GetByte(&buffer[7]);
			rchksum = GetByte(&buffer[9]+(linesize*2));
			chksum=linesize+(address/0x100)+(address%0x100)+recordtype+rchksum;

			if (recordtype==1) break; /*End of record*/

			for(j=0; j<linesize; j++)
			{
				value=GetByte(&buffer[9]+(j*2));
				chksum+=value;
				if((address+j)<memsize) FlashBuff[address+j]=value;
			}
			if(MaxAddress<(address+linesize-1)) MaxAddress=(address+linesize-1);

			if((chksum%0x100)!=0)
			{
				printf("ERROR: Bad checksum in file '%s' at line %d\r\n", filename, line_counter);
				return -1;
			}
		}
    }
    fclose(filein);

    return MaxAddress;
}

void GetNameFromPath(char * path, char * name)
{
	int i, j;

	for(i=0; path[i]!=0; i++);
	for(; (path[i]!='\\')&&(path[i]!='/')&&(i>=0); i--);
	for(j=0, i++; (path[i]!='.')&&(path[i]!=0); i++, j++) name[j]=path[i];
	name[j]=0;
}

int set_options (char * opt)
{
	int rvalue=0, unknown=0;
	char fusename[BUFFSIZE];
	int j, k;
	
	static char Help[] =
	"Usage: %s [-options] hex_file\n\n"
	"available options:\n"
#ifdef __unix__
	"p - Select port name. Example -p/dev/ttyUSB0.  No spaces between the 'p' and port name.\n"
#else
	"p - Select port number. Example -p5 for COM5.  No spaces between the 'p' and port number.\n"
#endif
	"r[filename] - Read Flash Memory. Example -rmyfile.hex.  No spaces between the 'r' and filename.\n"
	"v - Verify flash.\n"
#ifdef __unix__
	"s - Slow/Unbuffered transmission.\n"
#endif
	"f - Fuse value.  For example -fCS0=1. Valid fuses are (default in []):\n"
	"    CS0 [0], CS1 [0], SUT0 [1], SUT1 [1], Reset [1], BODEnable [1]\n"
	"    OCDDisable [1], ISP [1], UsrSigProDis [0], TristatePorts [0]\n"
	"    OCDInt [0], IAPDisable [0]\n"
	"c - Calibrate S2S Dongle\n"
	"k - Fill flash buffer with random values\n"
	"h - This help.\n\n";

	switch (opt[0])
	{
		default:
		    unknown=1;
		case 'h':
		case '?':
		    printf("%s: AT89LP Programmer Using S2S Dongle, Version %s\n", ProgName, version);
		    printf("by Jesus Calvino-Fraga, Copyright (c) 2012\n\n");
			if (unknown) printf("Unknown option: %c\n", opt[0]);
		    if (opt[0]=='v') exit(0);
			printf(Help, ProgName);
			exit(1);
		break;
		case 'p':
#ifdef __unix__
			strcpy(SerialPort, &opt[1]);
#else
			SerialPort=atoi(&opt[1]);
#endif
		break;
		case 'r':
			ReadFlash=1;
			strcpy(OutName, &opt[1]);
		break;
		case 'v':
			VerifyFlash=1;
		break;
		case 'c':
			Calibrate=1;
		break;
		case 'k':
			RandomValues=1;
		break;
#ifdef __unix__
		case 's':
			SlowTx=1;
		break;
#endif
		case 'f':
			// Get the fuse name
			for(j=1, k=0; (opt[j]!=0) && (opt[j]!='='); j++, k++)
			{
				fusename[k]=opt[j];
			}
			if(opt[j]=='=') j++;
			fusename[k]=0;
			printf("Fusename=%s\n", fusename);
			
			     if(stricmp(fusename, "CS0")==0) CS0=atoi(&opt[j]);
			else if(stricmp(fusename, "CS1")==0) CS1=atoi(&opt[j]);
			else if(stricmp(fusename, "SUT0")==0) SUT0=atoi(&opt[j]);
			else if(stricmp(fusename, "SUT1")==0) SUT1=atoi(&opt[j]);
			else if(stricmp(fusename, "Reset")==0) Reset=atoi(&opt[j]);
			else if(stricmp(fusename, "BODEnable")==0) BODEnable=atoi(&opt[j]);
			else if(stricmp(fusename, "OCDDisable")==0) OCDDisable=atoi(&opt[j]);
			else if(stricmp(fusename, "ISP")==0) ISP=atoi(&opt[j]);
			else if(stricmp(fusename, "UsrSigProDis")==0) UsrSigProDis=atoi(&opt[j]);
			else if(stricmp(fusename, "TristatePorts")==0) TristatePorts=atoi(&opt[j]);
			else if(stricmp(fusename, "OCDInt")==0) OCDInt=atoi(&opt[j]);
			else if(stricmp(fusename, "IAPDisable")==0) IAPDisable=atoi(&opt[j]);
			else
			{
				printf("Unknown fuse '%s'\n", fusename);
				exit(1);
			}
	
		break;
		
	}
	return (rvalue);
}

void ProcLineOptions (int argc, char **argv)
{
	int cont_par=0;
	int i, j;

	/*Get the program name*/
    GetNameFromPath(argv[0], ProgName);

	for (j=1; j<argc; j++)
	{
		if(argv[j][0]=='-')
		{
			set_options(&argv[j][1]);
		}
		else
		{
			switch(cont_par)
			{
				case 0:
 				   cont_par++;
                   strcpy(HexName, argv[j]);
				break;

				default:
					cont_par++;
				break;
			}
		}
	}
}

unsigned char flip (unsigned char val)
{
	unsigned char j=0;
	
	if(val & 0x80) j|=0x01;
	if(val & 0x40) j|=0x02;
	if(val & 0x20) j|=0x04;
	if(val & 0x10) j|=0x08;
	if(val & 0x08) j|=0x10;
	if(val & 0x04) j|=0x20;
	if(val & 0x02) j|=0x40;
	if(val & 0x01) j|=0x80;
	
	return j;
}

#ifdef __unix__
void Sleep (int msec)
{
	struct timespec req;

	req.tv_sec=0;
	req.tv_nsec=1000000L*msec;
    nanosleep (&req, NULL);
}

void Delay (int usec)
{
	struct timespec req;

	req.tv_sec=0;
	req.tv_nsec=1000L*usec;
    nanosleep (&req, NULL);
}

#endif

int SendBuff (unsigned char * tx, unsigned char * rx, int n)
{
	DWORD j;
	int bytes;
	int k;
	unsigned char ltx[BUFFSIZE];
	unsigned char lrx[BUFFSIZE];
	int to_return=0;
	
	if(n>BUFFSIZE) // Sanity check
	{
		printf("Local buffer is not big enough in SendBuff()\n");
		return -1;
	}
	
	// The bit order in RS232 is opposite to SPI, so flip the transmit buffer
	// before sending.
	for(k=0; k<n; k++) ltx[k]=flip(tx[k]);

	if(byte_offset==1) // Adjust preamble for the LP2051/LP4051
	{
		ltx[1]=flip(0xaa); 
	}

#ifdef __unix__

	{
		int k;
		time_t ltime1, ltime2;

		time(&ltime2);
	    ltime1=ltime2+3;
		j=0;
		while(j!=(n-byte_offset))
		{
			if(SlowTx==1) // Unbuffered/slow transmission
			{
				k=write(fd, &ltx[byte_offset+j], 1);
				Delay(180); // Needed if using the CH341 chipset 
			}
			else // Buffered/fast transmission
			{
				k=write(fd, &ltx[byte_offset+j], n-byte_offset);
			}
			if(k>0) j+=k;
			time(&ltime2);
			if(ltime2>=ltime1) 
		 	{
				printf("\nFailed in call to write(): %s\n", strerror(errno));
				return -1;
			}
	    }	
	}
#else
	// Perform a buffered transmission. Works great even for USB to RS232 adapters!
	if (!WriteFile(hComm, &ltx[byte_offset], n-byte_offset, &j, NULL))
	{
		printf("Failed in call to WriteFile()\n");
		return -1;
	}
#endif

	//Get the response.  We receive a byte for each byte transmitted.
	j=0;
	memset(lrx, 0, BUFFSIZE);
	memset(rx, 0, BUFFSIZE);
	
#ifdef __unix__
	{
		int k;
		time_t ltime1, ltime2;

		time(&ltime2);
	    ltime1=ltime2+3;
		
		while(j!=(n-byte_offset))
		{
			k = read(fd, &lrx[byte_offset+j], 1);
			if(k>0) j+=k;
			time(&ltime2);
			if(ltime2>=ltime1) 
		 	{
				break;
			}
	    }	
	}
#else
	if (!ReadFile(hComm, &lrx[byte_offset], n-byte_offset, &j, NULL))
	{
		printf("Failed in call to ReadFile()\n");
		return -1;
	}
#endif
	
	if(j!=(n-byte_offset))
	{
		//printf("Expected %d, received %d\n", n-byte_offset, j); // For debug only!
#ifdef __unix__
		tcflush(fd, TCIFLUSH);
#else
		FlushFileBuffers(hComm);		
#endif
		to_return=1;
	}
	
	// Also flip the received buffer
	for(k=0; k<n; k++) rx[k]=flip(lrx[k]);
	
	return to_return;
}

int SendBuffNoRx (unsigned char * tx, int n)
{
	DWORD j;
	int k;
	unsigned char ltx[BUFFSIZE];
	
	if(n>BUFFSIZE) // Sanity check
	{
		printf("Local buffer is not big enough in SendBuff()\n");
		return -1;
	}
	
	// The bit order in RS232 is opposite to SPI, so flip the transmit buffer
	// before sending.
	for(k=0; k<n; k++) ltx[k]=flip(tx[k]);
	
#ifdef __unix__
	k=write(fd, &ltx[byte_offset], n-byte_offset);
	Sleep(n);
	if(k<0)
	{
		printf("\nFailed in call to write(): %s\n", strerror(errno));
		return -1;
	}
#else
	// Perform a buffered transmission. Works great even for USB to RS232 adapters!
	if (!WriteFile(hComm, ltx, n, &j, NULL))
	{
		printf("Failed in call to WriteFile()\n");
		return -1;
	}
#endif

	return 0;
}


/* FYI RS-232 pinout:
DB9	DB25	Function
1	8	    Data carrier detect
2	3	    Receive data
3	2	    Transmit data
4	20   	Data terminal ready
5	7	    Signal ground
6	6	    Data set ready
7	4	    Request to send
8	5	    Clear to send
9	22	    Ring indicator
*/

#ifdef __unix__

// The the slave select (SS*) pin to either zero or one
void Set_SS_to (int x)
{
    int status;

    ioctl(fd, TIOCMGET, &status);

	if(x==0)
		status |= TIOCM_DTR;
	else
    	status &= ~TIOCM_DTR;

    ioctl(fd, TIOCMSET, &status);
}

// The the reset (RST*) pin to either zero or one
void Set_RST_to (int x)
{
    int status;

    ioctl(fd, TIOCMGET, &status);

	if(x==0)
		status |= TIOCM_RTS;
	else
    	status &= ~TIOCM_RTS;

    ioctl(fd, TIOCMSET, &status);
}

#else

// The the slave select (SS*) pin to either zero or one
void Set_SS_to (int x)
{
	EscapeCommFunction(hComm, (x==0)?SETDTR:CLRDTR);
}

// The the reset (RST*) pin to either zero or one
void Set_RST_to (int x)
{
	EscapeCommFunction(hComm, (x==0)?SETRTS:CLRRTS);
}

#endif

void ProgramInterfaceEnable (void)
{
	byte_offset=0;

	Set_RST_to(1);
	Sleep(100);
	Set_RST_to(0); // Put the at89lp into program mode (reset pin is zero)
	Set_SS_to(1);

	// Program Interface Enable (MUST be first command!) 10101100 01010011
	tx[0]=0xaa;
	tx[1]=0x55;
	tx[2]=0xac;
	tx[3]=0x53;
	tx[4]=0x00; // Send dummy low address byte. 53h should be returned on MISO if ISP is enabled.

	Set_SS_to(0);
	SendBuff(tx, rx, 5);
	Set_SS_to(1);

	if(rx[4]!=0x53)
	{
		// Maybe is a LP51/LP52...
		Set_RST_to(1);
		Sleep(100);
		Set_RST_to(0); // Put the at89lp into program mode (reset pin is zero)
		Set_SS_to(1);

		// Program Enable(1) 10101100 01010011 xxxxxxxx xxxxxxxx (0110 1001)(2)
		tx[0]=0xac;
		tx[1]=0x53;
		tx[2]=0x00;
		tx[3]=0x00; // Send dummy byte. 69h should be returned on MISO if ISP is enabled.

		Set_SS_to(0);
		SendBuff(tx, rx, 4);
		Set_SS_to(1);
		
		// For the LP51/LP52 a 69h is returned
		if(rx[3]!=0x69)
		{
			// Maybe is a LP2052/LP4052
			Set_RST_to(1);
			Sleep(100);
			Set_RST_to(0); // Put the at89lp into program mode (reset pin is zero)
			
			//Program Enable(1) 10101010 10101100 01010011
			tx[0]=0xaa;
			tx[1]=0xac;
			tx[2]=0x53;

			Set_SS_to(0);
			SendBuff(tx, rx, 3);
			Set_SS_to(1);

			//sprintf(feedback,"Received after program interface enable: %02x, %02x, %02x", rx[0], rx[1], rx[2]);
			//AfxMessageBox(feedback);
			byte_offset=1;
		}
		else
		{
			// It is either a LP51 or LP52.  Set buffer offset to 2.
			byte_offset=2;
		}
	}
	else
	{
		byte_offset=0;
	}
}

void BusyFlag (void)
{
    time_t ltime1, ltime2;

	time(&ltime2);
    ltime1=ltime2+10;

	rx[5]=0;
	while( ((rx[5]&0x01)==0) && (ltime2<ltime1) ) // Busy flag is zero when busy
	{
		// Read Status 01100000 xxxxxxxx xxxxxxxx Status Out		
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0x60; // 01100000
		tx[3]=0x00; // xxxxxxxx
		tx[4]=0x00; // xxxxxxxx
		tx[5]=0x00; // Status Out
		Set_SS_to(0);
		SendBuff(tx, rx, 6);
		Set_SS_to(1);
        
        time(&ltime2);
	}
	
}

int StartCommunication(void) 
{
    int portn;
	char feedback[0x300];
	int j, address;
	static int onetime=1;

	byte_offset=0;
	memset(tx, 0, 10);
	if (SendBuff(tx, rx, 10)!=0)
	{
		printf("ERROR: S2S Dongle not detected.\n");
		exit(-1);
	}

	ProgramInterfaceEnable();

	//Identify the microcontroller
	    
	// Read Atmel Signature Page(2)(6) 00111000 00000000 00bbbbbb DataOut 0 ... DataOut n
	address=0;
	tx[0]=0xaa;
	tx[1]=0x55;
	tx[2]=0x38;
	tx[3]=address/0x100; // 00000000
	tx[4]=address%0x100; // 00bbbbbb
	// The next bytes are don't care...
	
	Set_SS_to(0);
	SendBuff(tx, rx, 16+5); // Can't use buff_size here because we don't know which processor we are conecting to.
	Set_SS_to(1);

	//Save the first 16 bytes
	for(j=0; j<16; j++)
	{
		Atmel_Signature[j]=rx[j+5];
	}
	sprintf(ID_Str, "%02X%02X%02X", Atmel_Signature[0], Atmel_Signature[1], Atmel_Signature[2]);

	for(j=0; at_part_ID_S2S[j].size!=0; j++)
	{
		if( strcmp(ID_Str, at_part_ID_S2S[j].ID_Str)==0 )
		{
			memsize=at_part_ID_S2S[j].size;
			strcpy(Name_Str, at_part_ID_S2S[j].name);
			buff_size=at_part_ID_S2S[j].buff_size;
			if(onetime) // Shows the message just once.
			{
				printf("%s detected.\n", Name_Str);
				onetime=0;
			}
			break;
		}
	}
	
	if(at_part_ID_S2S[j].size==0)
	{
		printf("ERROR: Not a recognized AT89LP.\n");
	    Set_RST_to(1);
	    CloseSerialPort();
		exit(-1);
	}

	return 0;
}

int DoVerify(void)
{
	int j, k, address;	
	
	// Verify
	ProgramInterfaceEnable();

	printf("Verifying 0....10...20...30...40...50...60...70...80...90...100%%\n");
	printf("          ");
	
	for(address=0; address<memsize; address+=buff_size)
	{
		// Read Code Page 00110000 000aaaaa aabbbbbb DataOut 0 ... DataOut n
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0x30;
		tx[3]=address/0x100; // 000aaaaa
		tx[4]=address%0x100; // aabbbbbb
		memset(&tx[5], 0xff, buff_size);
		
		Set_SS_to(0);
		if(SendBuff(tx, rx, buff_size+5)!=0)
		{
			// Didn't receive the right number of bytes, try again just once more:
			// Flush the transmitter and receiver:
			Set_SS_to(1);
			Sleep(100);
			Set_SS_to(0);
			SendBuff(tx, rx, buff_size+5);
		}
		Set_SS_to(1);
	
		for(k=0; k<buff_size; k++)
		{
			if( ( (k+address) % (memsize/50) ) == 0) { printf("*"); fflush(stdout); }
			if(FlashBuff[address+k]!=rx[k+5])
			{
				printf("\nError at address %04XH\n", address+k);
				return 1;
			}
		}
	}
	printf("\nSuccess! Device memory matches buffer.\n");
	return 0;
}

void DoFlash(void) 
{
	int j, k, address;	

	#define LINESIZE 32

	if(StartCommunication()!=0)
	{
        return;
	}

	// Chip Erase 10001010
    printf("Erasing...");
	Sleep(500);
	if(byte_offset!=2)
	{
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0x8a;
		Set_SS_to(0);
		SendBuff(tx, rx, 3);
		Set_SS_to(1);
	}
	else // For the LP51/LP52
	{
		ProgramInterfaceEnable();
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0xac;
		tx[3]=0x80;
		tx[4]=0x00;
		tx[5]=0x00;
		Set_SS_to(0);
		SendBuff(tx, rx, 6);
		Set_SS_to(1);
	}
	BusyFlag();
    printf(" done.\n");

	// Write flash
	printf("Writing   0....10...20...30...40...50...60...70...80...90...100%%\n");
	printf("          ");
	for(address=0; address<memsize; address+=buff_size)
	{
		int DoSend;

		// Write Code Page 01010000 000aaaaa aabbbbbb DataIn 0 ... DataIn n
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0x50;
		tx[3]=address/0x100; // 000aaaaa
		tx[4]=address%0x100; // aabbbbbb
		
		for(k=0, DoSend=0; k<buff_size; k++)	
		{
			if( ( (k+address) % (memsize/50) ) == 0) { printf("*"); fflush(stdout); }
			tx[k+5]=FlashBuff[k+address];
			if(FlashBuff[k+address]!=0xff) DoSend=1;
		}
        
		if(DoSend)
		{
			Set_SS_to(0);
			SendBuff(tx, rx, buff_size+5);
			Set_SS_to(1);
		}
		BusyFlag();
	}
	printf("\n");
	
	// Verify that the Flash was written correctly...
	if (DoVerify())	return;

	printf("Writing fuses... ");

	//Write User Fuses 11100001 00000000 00bbbbbb DataIn 0 ... DataIn n
	//Write User Fuses with Auto-Erase 11110001 00000000 00bbbbbb DataIn 0 ... DataIn n
	tx[0]=0xaa;
	tx[1]=0x55;
	tx[2]=0xf1; // 11110001
	tx[3]=0x00; // 00000000
	tx[4]=0x00; // 00bbbbbb

	// Selects source for the system clock
	tx[5+0]=(CS0)?0xff:0x00;
	tx[5+1]=(CS1)?0xff:0x00;

	// Selects time-out delay for the POR/BOD/PWD wake-up period:
	tx[5+2]=(SUT0)?0xff:0x00;
	tx[5+3]=(SUT1)?0xff:0x00;
		
	if( (strcmp(Name_Str, "AT89LP428")==0)  ||
		(strcmp(Name_Str, "AT89LP828")==0)  ||
		(strcmp(Name_Str, "AT89LP3240")==0) ||
		(strcmp(Name_Str, "AT89LP6440")==0)) 
	{
		tx[5+4] =(Reset)?0xff:0x00;         // Reset Pin Enable
		tx[5+5] =(BODEnable)?0xff:0x00;     // Brown-out Detector Enable
		tx[5+6] =(OCDDisable)?0xff:0x00;    // On-chip Debug DISABLE
		tx[5+7] =(ISP)?0xff:0x00;           // ISP Enable
		tx[5+8] =(UsrSigProDis)?0xff:0x00;  // User Signature Programming
		tx[5+9] =(TristatePorts)?0xff:0x00; // Tristate Ports
		tx[5+10]=(OCDInt)?0xff:0x00;        // OCD Interface Select
		tx[5+11]=(IAPDisable)?0xff:0x00;    // In-Application Programming

		Set_SS_to(0);
		SendBuff(tx, rx, 5+12);
		Set_SS_to(1);
		BusyFlag();
	}
	else if ( (strcmp(Name_Str, "AT89LP51RD2")==0) ||
			  (strcmp(Name_Str, "AT89LP51ED2")==0) ||
			  (strcmp(Name_Str, "AT89LP51ID2")==0) )
	{
		// Selects source for the system clock
		tx[5+0]=(CS0)?0x00:0xff;
		tx[5+1]=(CS1)?0x00:0xff;
	
		// Selects time-out delay for the POR/BOD/PWD wake-up period:
		tx[5+2]=(SUT0)?0xff:0x00;
		tx[5+3]=(SUT1)?0xff:0x00;
	
		tx[5+4] =0xff;     // Bootloader jump bit.  ff:run user code
		tx[5+5] =0xff;     // External RAM Enable. ff: enabled.
		tx[5+6] =0x00;     // Compatibility mode. 00: single cycle fast mode
		tx[5+7] =(ISP)?0xff:0x00;           // ISP Enable
		tx[5+8] =0x00;     // X1/X2.  00: X2
		tx[5+9] =(OCDDisable)?0xff:0x00;    // On-chip Debug DISABLE
		tx[5+10] =(UsrSigProDis)?0xff:0x00;  // User Signature Programming
		tx[5+11]=(TristatePorts)?0xff:0x00; // Tristate Ports
		tx[5+12]=0x00;    // 00: EEPROM is not erased during chiperase
		tx[5+13]=0xff;
		tx[5+14]=0xff;
		tx[5+15]=0xff;
		tx[5+16]=0xff;
		tx[5+17]=0xff;
		tx[5+18]=0xff;
		
		Set_SS_to(0);
		SendBuff(tx, rx, 5+19);
		Set_SS_to(1);
		BusyFlag();
	
	}

	else if( (strcmp(Name_Str, "AT89LP213")==0)  ||
             (strcmp(Name_Str, "AT89LP214")==0)  ||
             (strcmp(Name_Str, "AT89LP216")==0) )
	{
		if((CS0==1)&&(CS1==0))
		{
			printf("WARNING: CS0=1 and CS1=0 is reserved in the %s.\r\n"
				   "Using internal oscillator instead.", Name_Str);
			tx[5+0]=0xff;
			tx[5+1]=0xff;
		}
		tx[5+4] =(Reset)?0xff:0x00;         // Reset Pin Enable
		tx[5+5] =(BODEnable)?0xff:0x00;     // Brown-out Detector Enable
		tx[5+6] =(OCDDisable)?0xff:0x00;    // On-chip Debug DISABLE
		tx[5+7] =(ISP)?0xff:0x00;           // ISP Enable
		tx[5+8]  = ( Atmel_Signature[8] & 0x01 )?0xff:0x00; // RC oscillator frequency adjustment
		tx[5+9]  = ( Atmel_Signature[8] & 0x02 )?0xff:0x00;
		tx[5+10] = ( Atmel_Signature[8] & 0x04 )?0xff:0x00;
		tx[5+11] = ( Atmel_Signature[8] & 0x08 )?0xff:0x00;
		tx[5+12] = ( Atmel_Signature[8] & 0x10 )?0xff:0x00;
		tx[5+13] = ( Atmel_Signature[8] & 0x20 )?0xff:0x00;
		tx[5+14] = ( Atmel_Signature[8] & 0x40 )?0xff:0x00;
		tx[5+15] = ( Atmel_Signature[8] & 0x80 )?0xff:0x00;
		tx[5+16] =(UsrSigProDis)?0xff:0x00;  // User Signature Programming
		tx[5+17] =(TristatePorts)?0xff:0x00; // Tristate Ports
		tx[5+18]=(OCDInt)?0xff:0x00;         // OCD Interface Select
		tx[5+18]=(IAPDisable)?0xff:0x00;     // In-Application Programming

		Set_SS_to(0);
		SendBuff(tx, rx, 19+5);
		Set_SS_to(1);
		BusyFlag();
	}
	else if( (strcmp(Name_Str, "AT89LP51")==0)  ||
             (strcmp(Name_Str, "AT89LP52")==0) )
	{
		// The protocol for the LP51/LP52 is too different from the ones
		// above, so start over...
		ProgramInterfaceEnable();

		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0x71; // 01110001
		tx[3]=0x00; // 00000000
		tx[4]=0x00; // 00bbbbbb

		// Selects source for the system clock
		tx[5+0]=(CS0)?0xff:0x00;
		tx[5+1]=(CS1)?0xff:0x00;

		// Selects time-out delay for the POR/BOD/PWD wake-up period:
		tx[5+2]=(SUT0)?0xff:0x00;
		tx[5+3]=(SUT1)?0xff:0x00;

		tx[5+4] = 0x00;                     // CPU in fast mode
		tx[5+5] =(ISP)?0xff:0x00;           // ISP Enable
		tx[5+6] =(UsrSigProDis)?0xff:0x00;  // User Signature Programming
		tx[5+7] =(TristatePorts)?0xff:0x00; // Tristate Ports
		tx[5+8] =(IAPDisable)?0xff:0x00;    // In-Application Programming
		tx[5+9] = 0xff;                     // 5M resistor on XTAL1 Disabled
		
		for(j=10; j<64; j++) tx[5+j]=0;

		Set_SS_to(0);
		SendBuff(tx, rx, 5+64);
		Set_SS_to(1);
		BusyFlag();
	}
	else if( (strcmp(Name_Str, "AT89LP2052")==0)  ||
             (strcmp(Name_Str, "AT89LP4052")==0) )
	{
		// Write User Fuses 10101010 11100001 xxxxxxxx xxxx xxx xxxxFFFF
		tx[0]=0xaa;
		tx[1]=0x55;
		tx[2]=0xE1; // 11100001
		tx[3]=0x00; // 00000000
		tx[4]=0x00; // 00000000

		tx[5]=0x0e; // Keep ISP enabled: bit0 = 0
		
		Set_SS_to(0);
		SendBuff(tx, rx, 6);
		Set_SS_to(1);
		BusyFlag();
	}
	else
	{
		printf("WARNING: Don't know how to write fuses for the %s\n", Name_Str);
	}
	printf("Done.\n");
    printf("Flash program command completed.\n");
}

int main(int argc, char **argv)
{
	char c;
	unsigned int j, k;
	unsigned int address;
	int iserror=0;
	
#ifndef __unix__
	SetConsoleTitle("AT89LP Programmer using S2S Dongle");
#endif

	ProcLineOptions (argc, argv);

	if(OpenSerialPort(SerialPort, 115200, NOPARITY, 8, TWOSTOPBITS)!=0)
	{
#ifdef __unix__
		printf("Couldn't open serial port %s\n", SerialPort);
#else
		printf("Couldn't open serial port COM%d\n", SerialPort);
#endif
		exit(-1);
	}
	else
	{
#ifdef __unix__
		printf("Using %s\n", SerialPort);
#else
		printf("Using COM%d\r\n", SerialPort);
#endif
	}
	
	if(Calibrate==1)
	{
		printf("\nCalibrating big pulse.\n"
		       "Adjust R1 until pins 1 and 4 of the 74HC221 IC are equal.\n"
		       "Press any key when done.\n");
		memset(tx, 0, BUFFSIZE);
		while(kbhit()==0) SendBuffNoRx(tx, BUFFSIZE);
		while(kbhit()) getch();
	
		printf("\nCalibrating small pulse.\n"
		       "Adjust R3 until pins 1 and 12 of the 74HC221 IC are equal.\n"
		       "Press any key when done.\n");
		memset(tx, 0xff, BUFFSIZE);
		while(kbhit()==0) SendBuffNoRx(tx, BUFFSIZE);
		while(kbhit()) getch();
		
		printf("\nCalibrating frequency.\n"
		       "Adjust R7 until pin 3 of the LM555 IC shows 115.2 kHz.\n"
		       "Press any key to finish.\n");
		memset(tx, 0xaa, BUFFSIZE);
		while(kbhit()==0) SendBuffNoRx(tx, BUFFSIZE);
		while(kbhit()) getch();
		
		CloseSerialPort();
		
		return 0;
	}
	
	if(RandomValues==1)
	{
		int j;

		printf("Loading buffer with random values... ");		
		srand (time(NULL));
		for(j=0; j<0x10000; j++)
		{
			FlashBuff[j]=rand()%0x100;
		}
		printf("done\n");
		// Put a ljmp 0x0000 at the reset start so no random code is executed
		FlashBuff[0]=0x02;
		FlashBuff[1]=0x00;
		FlashBuff[2]=0x00;
	}

	StartCommunication();
	
	if(ReadFlash==1)
	{
		FILE * fileout;
		
		if(strlen(OutName)>0)
		{
			if ( (fileout=fopen(OutName, "w")) == NULL )
			{
				printf("Error: Can't create file `%s`.\r\n", OutName);
				exit(-1);
			}
		}
		else
		{
			fileout=stdout;
		}

		for(address=0; address<memsize; address+=16)
		{
			int chksum;
			// Read Code Page(2) 00110000 000aaaaa aabbbbbb DataOut 0 ... DataOut n
			tx[0]=0xaa;
			tx[1]=0x55;
			tx[2]=0x30;
			tx[3]=address/0x100; // 000aaaaa
			tx[4]=address%0x100; // aabbbbbb
			// The next 16 bytes are don't care...
			
			Set_SS_to(0);
			if(SendBuff(tx, rx, 16+5)!=0)
			{
				// Didn't receive the right number of bytes, try again once more
				Set_SS_to(1);
				Sleep(100);
				Set_SS_to(0);
				if(SendBuff(tx, rx, 16+5)!=0)
				{
					fclose(fileout);
					Set_RST_to(1);
					CloseSerialPort();
					printf("Error reading from S2S dongle\n");
					return(-1);
				}
			}
			Set_SS_to(1);
		
			fprintf(fileout, ":10%04X00", address);
			chksum=0x10+address/0x100+address%0x100;
			for(k=0; k<16; k++)
			{
				chksum+=rx[k+5];
				fprintf(fileout, "%02X", rx[k+5]);
			}
			fprintf(fileout, "%02X\n", (0x100-(chksum&0xff))&0xff);
		}
		fprintf(fileout, ":00000001FF\n");
		
		if(strlen(OutName)>0) fclose(fileout);
	}
	
	else if( (strlen(HexName)>0) || (RandomValues==1) )
	{
    	time_t ltime1, ltime2;
	
		if((strlen(HexName)>0))
		{
		    printf("Loading file...");
		    ReadHexFile(HexName);
		    printf(" done.\n");
	    }
    
		time(&ltime1);
		if(VerifyFlash==0)
		{
			DoFlash();	
		}
		else
		{
			DoVerify();
		}
		time(&ltime2);
		printf("Elapsed time: %d seconds\n", (int) (ltime2-ltime1) );
	}
	else
	{
		printf("Atmel Signature Page (first 16 bytes only):\n");
		printf("%04XH: ", 0);
		for(k=0; k<16; k++)
		{
			printf("%02XH ", Atmel_Signature[k]);
		}
		printf("\n");

		printf("Fuses:\n");
		for(address=0; address<0x10; address+=16)
		{
		    // Read User Fuses(2)(3)(4) 01100001 00000000 00bbbbbb DataOut 0 ... DataOut n
			tx[0]=0xaa;
			tx[1]=0x55;
			tx[2]=0x61;
			tx[3]=address/0x100; // 00000000
			tx[4]=address%0x100; // 00bbbbbb
			// The next 16 bytes are don't care...
			
			Set_SS_to(0);
			SendBuff(tx, rx, 16+5);
			Set_SS_to(1);
		
			printf("%04XH: ", address);
			for(k=0; k<16; k++)
			{
				printf("%02XH ", rx[k+5]);
			}
			printf("\n");
		}
	}

	Set_RST_to(1);
	CloseSerialPort();
	
	return 0;
}
