/*
	SPI communication with MCP3004 using the S2S Dongle
	
	Jesus Calvino-Fraga 2012, jesusc (at) ece.ubc.ca
	   
	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2, or (at your option) any
	later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <windows.h>

#define BANNER "\nSPI communication with MCP3004 using the S2S Dongle\n\n"

#define EQ(X,Y) (_stricmp(X, Y)==0)
#define NEQ(X,Y) (_stricmp(X, Y)!=0)

HANDLE hComm=INVALID_HANDLE_VALUE;
int SerialPort=1;

int OpenSerialPort (int Port, DWORD baud, BYTE parity, BYTE bits, BYTE stop)
{
	char sPort[16];
	DCB dcb;
	COMMTIMEOUTS Timeouts;

	sprintf(sPort, "\\\\.\\COM%d", Port);
	
	hComm = CreateFile(sPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hComm == INVALID_HANDLE_VALUE)
	{
		printf("Failed to open up the comunication port COM%d\n", Port);
		return -1;
	}
	
	if (!GetCommState(hComm, &dcb))
	{
		printf("Failed in call to GetCommState()\n");
		return -1;
	}

	dcb.fAbortOnError=FALSE;
	dcb.BaudRate = baud; 
	dcb.Parity = parity;
	dcb.ByteSize = bits;
	dcb.StopBits = stop;
	dcb.fDsrSensitivity = FALSE;
	dcb.fOutxCtsFlow = FALSE;
	dcb.fOutxDsrFlow = FALSE;
	dcb.fOutX = FALSE;
	dcb.fInX = FALSE;

	//Now that we have all the settings in place, make the changes
	if (!SetCommState(hComm, &dcb))
	{
		printf("Failed in call to SetCommState()\n");
		return -1;
	}	

	//The always important timeouts.  May need to increase them for
	//USB to rs-232 adapters.

	ZeroMemory(&Timeouts, sizeof(COMMTIMEOUTS));
	Timeouts.ReadIntervalTimeout = 250;
	Timeouts.ReadTotalTimeoutMultiplier = 10;
	Timeouts.ReadTotalTimeoutConstant = 250;
	Timeouts.WriteTotalTimeoutMultiplier = 10;
	Timeouts.WriteTotalTimeoutConstant = 250;
	if (!SetCommTimeouts(hComm, &Timeouts))
	{
		printf("Failed in call to SetCommTimeouts\n");
		return -1;
	}

	return 0;
}

int CloseSerialPort (void)
{
	BOOL bSuccess;
	bSuccess = CloseHandle(hComm);
    hComm = INVALID_HANDLE_VALUE;
    if (!bSuccess)
    {
		printf("Failed to close serial port, GetLastError:%d\n", GetLastError());
		return -1;
	}
	return 0;
}

unsigned char flip (unsigned char val)
{
	unsigned char j=0;
	
	if(val & 0x80) j|=0x01;
	if(val & 0x40) j|=0x02;
	if(val & 0x20) j|=0x04;
	if(val & 0x10) j|=0x08;
	if(val & 0x08) j|=0x10;
	if(val & 0x04) j|=0x20;
	if(val & 0x02) j|=0x40;
	if(val & 0x01) j|=0x80;
	
	return j;
}

int SendBuff (unsigned char * tx, unsigned char * rx, int n)
{
	#define BUFFSIZE 0x100
	DWORD j, k ;
	unsigned char ltx[BUFFSIZE];
	unsigned char lrx[BUFFSIZE];
	
	if(n>(BUFFSIZE-1))
	{
	}
	
	// Flip the transmit buffer before transmission
	for(k=0; k<n; k++)
	{
		ltx[k]=flip(tx[k]);
	}

	if (!WriteFile(hComm, ltx, n, &j, NULL))
	{
		printf("Failed in call to WriteFile()\n");
		return -1;
	}

	//Get the response
	j=0;
	if (!ReadFile(hComm, lrx, n, &j, NULL))
	{
		printf("Failed in call to ReadFile()\n");
		return -1;
	}
	
	if(j!=n)
	{
		printf("Expected %d, received %d\n", n, j);
		return -1;
	}
	
	// Flip the receive buffer after reception
	for(k=0; k<n; k++)
	{
		rx[k]=flip(lrx[k]);
	}
	
	return 0;
}


void Set_SS_to (int x)
{
	if(x==0)
	{
		EscapeCommFunction(hComm, SETDTR); // Make SS=0
	}
	else
	{
		EscapeCommFunction(hComm, CLRDTR); // Make SS=1
	}
}

int main(int argc, char **argv)
{
	char c;
	unsigned char tx[10], rx[10];
	unsigned int k;
	float ch[4];
	
	SetConsoleTitle("MCP3004 using S2S Dongle");
	printf(BANNER);
			
	if(argc>1)
	{
		if ( (argv[1][0]=='-') && (toupper(argv[1][1])=='p') )
		{
			SerialPort=atoi(&argv[1][2]);
		}
	}
	
	if(OpenSerialPort(SerialPort, 115200, NOPARITY, 8, TWOSTOPBITS)!=0)
	{
		printf("Couldn't open serial port COM%d\n", SerialPort);
		exit(-1);
	}
	else
	{
		printf("Using COM%d, press any key to finish.\n", SerialPort);
	}
	

	while(kbhit()==0)
	{
		int j;
		
		// Read the 4 channels of the MCP3004 one at a time
		for(j=0; j<4; j++)
		{
			// To see why the transmit buffer is set this way, take a look at
			// figure 6-1 in the MCP3004 datasheet available at
			// http://ww1.microchip.com/downloads/en/DeviceDoc/21295C.pdf
			
			tx[0]=0x01; tx[1]=0x80+j*0x10; tx[2]=0x55;
			
			Set_SS_to(0);
			if(SendBuff(tx, rx, 3)!=0)
			{
				CloseSerialPort();
				exit(1);
			}
			Set_SS_to(1);
			
			k=((int)(rx[1]&0x03)*0x100)+(int)(rx[2]);
			ch[j]=(k*3.3)/1024.0;
		}
		
		printf("V0=%5.3f V1=%5.3f V2=%5.3f V3=%5.3f\r", ch[0], ch[1], ch[2], ch[3]);
	}
	
	CloseSerialPort();
	printf("\n");
	
	return 0;
}
