
;***************************************;***************************************
;
;   Author              :	jeff bachiochi
;   Company             :  	imagine that
;	Web Site			:	www.imaginethatnow.com
;   Filename            :  	Midi.s 
;   Date                :  	1/29/2013
;   File Version        :  	1.2                                                   
;
;***************************************;***************************************
;{
;	Version 1.0	=	Part 1 Midi column (Midi Monitor)
;					monitors Midi messages on UART1,
;					sends text interpretations out UART2
;
;	Version 1.2	=	V1.0 plus Part 2 Midi column (Midi Drum Kit)
;					sends Midi messages on UART1,
;					sends text messages out UART2 and uses UART2 to configure 
;					the Drum Kit
;		Px<cr>	=	set pad number 1-8
;		Cx<cr>	=	set channel number 1-16
;		Vx<cr>	=	set velocity number 0-127
;		Ix<cr>	=	set instrumemt number 0-127
;		Nx<cr>	=	set pad note number 0-127
;		Sx<cr>	=	save pad configuration
;		Tx<cr>	=	trigger pad number 1-8
;		Bx<cr>	=	beats number 30-180
;		+	=	increment last command + reppeat
;		- 	=	decrement last command + reppeat
;		<cr>	=	repeat last command
;
;	Version 1.2	=	V1.1 plus LCD/button (Midi Drum Kit)
;					LCD/button interface makes it easy to select parameters
;					without a terminal interface
;
;	JP2(RA0)	=	jumper ON selects monitor
;	JP3(RA1)	=	jumper ON disables sending 'Midi Clock'
;	JP4	=	jumper selects Midi-Out/Midi-Thru
;
;	Digital Triggers
;	RA3, 4, 6, 7, RB8, 9, 10, 11					
;
;	Analog Triggers
;	RA0, 1, RB3, 4, 12, 13, 14, 15
;
;   Additional Notes:                         
;                                        
;                                             
	.equ	InvertUART1,	0			; 0=normal, 1=invert
;	31250 baud = BRG=128(@8MHz,BRGH=1)
	.equ	InvertUART2,	1			; 0=normal, 1=invert
;	115200 baud = BRG=8(@8MHz,BRGH=1)
	.equ	Debug,	0					; 0=no debug. 1=debug
	.equ	JP2,	1					; 0=monitor, 1=DrumKit
	.equ	JP3,	1					; 0=disable, 1=enable Midi Clock out
	.equ	TriggerType,	0			; 0=analog,	1=digital
;
;
;
;	Revision 4 chips have a bug in the TX FIFO
;}
;***************************************;***************************************
;
        .equ __24FV32KA302, 1
        .include "p24FV32KA302.inc"
;
;***************************************;***************************************
;
;	Configuration bits:
;
;***************************************;***************************************
;{
        config __FOSCSEL, FNOSC_FRCDIV & SOSCSRC_DIG  ; use the Internal Clock as the
                                        ; system clock
                                        
        config __FOSC, OSCIOFNC_OFF  	; no clock out                                      

        config __FWDT, FWDTEN_OFF  & WINDIS_OFF	; Turn off Watchdog Timer

        config __FPOR,  BORV_V30 & PWRTEN_ON & MCLRE_ON
                                        ; Set Brown-out Reset voltage and
                                        ; and set Power-up Timer to 16msecs
                                            
        config __FGS, GSS0_OFF        	; Set Code Protection Off for the 
                                     	       ;General Segment
		config __FICD,	ICS_PGx3		; Use 
;}
;***************************************;***************************************
;
;	Program Specific Constants (literals used in code)
;
;***************************************;***************************************
;{
;	PORTA
	.equ	RA0,	0					; AN0
	.equ	IN1,	0					; Trigger1
	.equ	RA1,	1					; AN1
	.equ	IN2,	0					; Trigger2
	.equ	RA2,	2					; AN13
	.equ	IN7,	2					; Trigger7	
	.equ	RA3,	3					; AN14
	.equ	IN8,	3					; Trigger8	
	.equ	LED,	4
;	.equ	RA5,	5
;	.equ	RA6,	6
	.equ	R_WR,	7
;
;	PORTB
	.equ	TX2,	0
	.equ	RX2,	1
	.equ	RX1,	2
	.equ	RS,	3
	.equ	E,	4
	.equ	RB5,	5
	.equ	RB6,	6
	.equ	TX1,	7
	.equ	D4,	8
	.equ	D5,	9
	.equ	D6,	10
	.equ	D7,	11
	.equ	RB12,	12					; AN12
	.equ	IN6,	12					; Trigger6	
	.equ	RB13,	13					; AN11
	.equ	IN5,	13					; Trigger5	
	.equ	RB14,	14					; AN10
	.equ	IN4,	14					; Trigger4	
	.equ	RB15,	15					; AN9
	.equ	IN3,	15					; Trigger3	
;
;	Flags 
	.equ	U1RXEmpty,	0				; 1=U1RXBuffer is empty
	.equ	U2RXEmpty,	0				; 1=U2RXBuffer is empty
	.equ	U1RXCharacterReady,	2		; 1=Character retireved from U1RXBuffer
	.equ	U2RXCharacterReady,	3		; 1=Character retireved from U2RXBuffer
	.equ	Request,	4				; 1=Good User Request
	.equ	ActiveSenseMessage,	5		; 1=SendActiveSenseMessage
	.equ	MidiClockMessage,	6		; 1=SendMidiClockMessage
	.equ	RB5State,	7				; 0(jumper on)=monitor,1(jumper off)=DrumKit
	.equ	RB6State,	8				; 0(jumper on)=no send Midi Clock,1(jumper off)=send Midi Clock		
	.equ	LCDFound,	9				; 1=LCD Found
;
    .equ 	UARTBuffer_Length, 64  		; Buffer Size
    .equ	DisplayWidth,	20			; Number of Characters in LCD's row
    .equ	Busy,	7					; MSB (7) is busy bit
    .equ	KeyCount,	20				; number of consecutive counts before good
    .equ	Row1Column1,	0x80		
    .equ	Row2Column1,	0xC0
    .equ	Row3Column1,	0x94
    .equ	Row4Column1,	0xD4
    .equ	MidiClockBeat,	24
    .equ	Min_OFF,	8
;}
;***************************************;***************************************
;
;	Global Declarations:
;
;***************************************;***************************************
;{
        .global	_wreg_init       		; Provide global scope to _wreg_init 
                                 		; routine. In order to call this routine
                                 		;  from a C file, place "wreg_init" in 
                                 		; an "extern" declarationin the C file.

        .global __reset          		; The label for the first line of code. 

        .global __T1Interrupt    		; Declare Timer 1 ISR name global
        .global __U1TXInterrupt    		; Declare UART1 TX ISR name global
        .global __U1RXInterrupt    		; Declare UART1 RX ISR name global  
        .global __U2TXInterrupt    		; Declare UART2 TX ISR name global
        .global __U2RXInterrupt    		; Declare UART2 RX ISR name global 
        .global __T2Interrupt    		; Declare Timer2 ISR name global  
        .global __T4Interrupt    		; Declare Timer4 ISR name global 
        .global __T5Interrupt    		; Declare Timer5 ISR name global        
        .global __CNInterrupt			; Declare Change Notice ISR as global 
        .global __ADC1Interrupt			; Declare A/D ISR as global         
;}
;***************************************;***************************************
;
;	Global Declarations:
;
;***************************************;***************************************
;{
        .section 	.nbss, bss, near
U1TXBuffer_Head: 	.space 	2   		; U1TXBuffer Pointer (in) 
U1TXBuffer_Tail: 	.space 	2   		; U1TXBuffer Pointer (out)
U1RXBuffer_Head: 	.space 	2   		; U1RXBuffer Pointer (in) 
U1RXBuffer_Tail: 	.space 	2   		; U1RXBuffer Pointer (out)
U2TXBuffer_Head: 	.space 	2   		; U2TXBuffer Pointer (in) 
U2TXBuffer_Tail: 	.space 	2   		; U2TXBuffer Pointer (out)
U2RXBuffer_Head: 	.space 	2   		; U2RXBuffer Pointer (in) 
U2RXBuffer_Tail: 	.space 	2   		; U2RXBuffer Pointer (out)
Flags:				.space	2			; application flags
MidiBeatNumber:		.space	2
TransposeNumber:	.space	2
LastPortA:			.space	2
LastPortB:			.space	2
ThisPortA:			.space	2
ThisPortB:			.space	2
TEMP:				.space	2
LCDBufferPointer:	.space	2
P1:					.space 	4			; channel, note, voice, notused
P2:					.space 	4
P3:					.space 	4
P4:					.space 	4
P5:					.space 	4
P6:					.space 	4
P7:					.space 	4
P8:					.space 	4
;
U1TXCharacter:	.space	1
U1RXCharacter:	.space	1
U2TXCharacter:	.space	1
U2RXCharacter:	.space	1
MessageNumber:	.space	1
MessageDataCount:	.space	1
NoteNumber:		.space	1
VelocityNumber:	.space	1
ChannelNumber:	.space	1
PressureNumber:	.space	1
ControllerNumber:	.space	1
ValueNumber:	.space	1
ProgramNumber:	.space	1
SongNumber:		.space	1
TimeCodeNumber:	.space	1
TempW:			.space	1
MyPad:			.space	1
MyChannel:		.space	1
MyNote:			.space	1
MyInstrument:	.space	1
MyVelocity:		.space	1
MyRequest:		.space	1
MyValue:		.space	1
MyBeats:		.space	1
LCDData:		.space	1
LCDRow:			.space	1
StringOffset:	.space	1
Key8:			.space	1
Key9:			.space	1
Key10:			.space	1
Key11:			.space	1
MyMode:			.space	1
MidiClockCount:	.space	1
In1Max:			.space	1
In2Max:			.space	1
In3Max:			.space	1
In4Max:			.space	1
In5Max:			.space	1
In6Max:			.space	1
In7Max:			.space	1
In8Max:			.space	1
DirFlags:		.space	1
TriggerFlags:	.space	1
;
U1TXBuffer:			.space	UARTBuffer_Length	; U1TXBuffer 
U1RXBuffer:			.space	UARTBuffer_Length	; U1RXBuffer
U2TXBuffer:			.space	UARTBuffer_Length	; U2TXBuffer 
U2RXBuffer:			.space	UARTBuffer_Length	; U2RXBuffer
LCDBuffer:			.space	20			; Message Buffer
TopOfMemoryUsage:	.space	1
;}
;***************************************;***************************************
;
;	EEPROM
;
;	Initializing data for Pad 1-8
;	channel, note, instrument, velocity
;
;***************************************;***************************************
;{
	.section	*,eedata
MyEEData:
 	.byte 10, 35, 14, 64
 	.byte 10, 38, 14, 64	
 	.byte 10, 40, 14, 64
 	.byte 10, 41, 14, 64 
 	.byte 10, 42, 14, 64
 	.byte 10, 50, 14, 64 	
 	.byte 10, 51, 14, 64
 	.byte 10, 49, 14, 64 		 	
;}
;***************************************;***************************************
;
;	Code Section in Program Memory
;
;***************************************;***************************************
;{
.text                             		; Start of Code section
__reset:
        mov 	#__SP_init,	W15       	; Initalize the Stack Pointer
        mov 	#__SPLIM_init,	W0     	; Init the Stack Pointer Limit Register
        mov 	W0, SPLIM
        nop                       		; Add NOP to follow SPLIM initialization
;        
        call	_wreg_init           	; Call _wreg_init subroutine
;***************************************
init_CLKDIV:
		mov		#0x3000,	W0			; 1:1 (8MHz clk)
		mov		W0,	CLKDIV
;***************************************		
init_PORTA:
	.if	TriggerType==0					; RA3:0=Analog
 		mov		#0x000F,	W0			; 1=analog, 0=digital
 										; 15:8=01100000, 7:0=00000011	
	.else								; All=Digital
 		mov		#0x0000,	W0			; 1=analog, 0=digital
 										; 15:8=00000000, 7:0=00000000 
 	.endif											
 		mov		W0,	ANSA       			; analog/digital
; 		
 		mov		#0x0000,	W0			; 1=open drain, 0=digital
 		 								; 15:8=0, 7:0=0 
 		mov		W0,	ODCA       			; open drain/digital
; 		
 		mov		#0xFF6F,	W0			; 1=input, 0=output
 										; 15:8=FF, 7:0=01101111 
 										; RA7=O, RA4=O
 		mov		W0,	TRISA       		; input/output
; 		 		 
 		mov		#0x0090,	W0			; 1=high, 0=low
 										; 15:8=0, 7:0=10010000 
 										; RA7=1, RA4=1		
 		mov		W0,	LATA       			; pin state
; 
	.if	TriggerType==0					; Analog (no IOC)
 		mov		#0x0000,	W0			; 1=Interrupt On Change, 0=no IOC
 										; 15:8=00000000, 7:0=00000000
 										; RB12=CN14, RB13=CN13, RB14=CN12, RB15=CN11,
 										; RA1=CN3,RA0=CN2	
	.else		 		 				; Digital (IOC)
 		mov		#0x780F,	W0			; 1=Interrupt On Change, 0=no IOC
 										; 15:8=01111000, 7:0=00001111
 										; RB12=CN14, RB13=CN13, RB14=CN12, RB15=CN11,
 										; RA1=CN3,RA0=CN2 
	.endif 			
 		mov		W0,	CNEN1
; 		 		 
;
	.if	TriggerType==0					; Analog (no pull-ups)
 		mov		#0x8000,	W0			; 1=PullUp enabled, 0=no PullUp
 										; 15:8=10000000, 7:0=00000000
 										; RB11=CN15, RB12=CN14, RB13=CN13, RB14=CN12,
 										; RB15=CN11, RA1=CN3,RA0=CN2	
 	.else										
 		mov		#0xF80C,	W0			; 1=PullUp enabled, 0=no PullUp
 										; 15:8=11111000, 7:0=00001100
 										; RB11=CN15, RB12=CN14, RB13=CN13, RB14=CN12,
 										; RB15=CN11, RA1=CN3,RA0=CN2
 	.endif											
 		mov		W0,	CNPU1 		
;***************************************		
init_PORTB:
	.if	TriggerType==0					; RB13:10=Analog
 		mov		#0xF000,	W0			; 1=analog, 0=digital
 										; 15:8=11110000, 7:0=00000000	
	.else								; All=Digital
 		mov		#0x0000,	W0			; 1=analog, 0=digital
 										; 15:8=00000000, 7:0=00000000 
 										; TX1=0, RB4=O, RB3=O, RX1=0, RX2=0, TX2=0
	.endif 												
 		mov		W0,	ANSB       			; analog/digital
; 		
 		mov		#0x0000,	W0			; 1=open drain, 0=digital
 										; 15:8=0, 7:0=0 
 										; TX1=0, RB4=O, RB3=O, RX1=0, RX2=0, TX2=0		
 		mov		W0,	ODCB       			; open drain/digital
; 		
 		mov		#0xFF66,	W0			; 1=input, 0=output
 										; 15:8=FF, 7:0=01100110, 
 										; TX1=0, RB4=O, RB3=O, RX1=1, RX2=1, TX2=0
 		mov		W0,	TRISB       		; input/output 		 
; 		
 		mov		#0x0018,	W0			; 1=high, 0=low
 										; 15:8=0, 7:0=00011000
 										; TX1=0, RB4=1, RB3=1, RX1=0, RX2=0, TX2=0
		mov		W0,	LATB 										
; 		 		 
	.if	TriggerType==0					; Analog (no IOC)
 		mov		#0x0000,	W0			; 1=Interrupt On Change, 0=no IOC
 										; CN31:22=00000000, CB21:16=00000000
 										; RA2=CN30, RA3=CN29 	
	.else
 		mov		#0x6001,	W0			; 1=Interrupt On Change, 0=no IOC
 										; CN31:22=01100000, CB21:16=00000001
 										; RA2=CN30, RA3=CN29, RB10=CN16 
 	.endif											
 		mov		W0,	CNEN2
; 	
	.if	TriggerType==0					; Analog (no pull-ups)
 		mov		#0x0960,	W0			; 1=PullUp enabled, 0=no PullUp
 										; CN31:24=00001001, CB23:16=01100000
 										; RA2=CN30, RA3=CN29, RB5=CN27, RB6=CN24,
 										; RB8=CN22, RB9=CN21, RB10=CN16	
	.else	 		 
 		mov		#0x6961,	W0			; 1=PullUp enabled, 0=no PullUp
 										; CN31:24=01101001, CB23:16=01100001
 										; RA2=CN30, RA3=CN29, RB5=CN27, RB6=CN24,
 										; RB8=CN22, RB9=CN21, RB10=CN16
	.endif 										  		
 		mov		W0,	CNPU2
;
 	.if InvertUART1==1
 		bclr	W0,	#TX1
 	.endif
; 	
 	.if InvertUART2==1
 		bclr	W0,	#TX2
 	.endif 
; 		 		
 		mov		W0,	LATB       			; pin state 
;***************************************
init_Varaibles:
		mov		#U1TXBuffer_Head,	W0
		mov		#TopOfMemoryUsage,	W1
;
init_Varaibles_Loop:
		clr.b	[W0++]
		cpseq	W0,		W1
		goto	init_Varaibles_Loop
;		
		mov		#U1TXBuffer,	W0
		mov		W0,	U1TXBuffer_Head
		mov		W0,	U1TXBuffer_Tail	
		mov		#U1RXBuffer,	W0
		mov		W0,	U1RXBuffer_Head
		mov		W0,	U1RXBuffer_Tail	
		mov		#U2TXBuffer,	W0
		mov		W0,	U2TXBuffer_Head
		mov		W0,	U2TXBuffer_Tail	
		mov		#U2RXBuffer,	W0
		mov		W0,	U2RXBuffer_Head
		mov		W0,	U2RXBuffer_Tail	
;
		mov		#tblpage(MyEEData),	W0
		mov		W0,		_TBLPAG			; Set up a pointer to the EEPROM
		mov		#tbloffset(MyEEData), W0
		mov		#P1,	W2
		mov		#16,	W1
;
EEPROM_Loop:
		tblrdl,	[W0++],	[W2++]			; Read the EEPROM data
		dec.b	W1,		W1
		cp0.b	W1
		bra		nz,		EEPROM_Loop
;					
		mov		#60,	W0				; initialize BPM to 60
		mov.b	WREG,	MyBeats
		mov		W0,		W2
		mov		#39360,	W0
		repeat	#17
		div.uw	W0,		W2
		mov		W0,		PR4
;
		mov		#1,		W0
		mov.b	WREG,	MyPad
		mov		#P1,	W1
		mov.b	[W1++],	W0
		mov.b	WREG,	MyChannel
		mov.b	[W1++],	W0
		mov.b	WREG,	MyNote						
		mov.b	[W1++],	W0
		mov.b	WREG,	MyInstrument	
		mov.b	[W1++],	W0
		mov.b	WREG,	MyVelocity			
;***************************************
init_jumpers:
	.if Debug==0
		btsc	PORTB,	#RB5			; skip if RB5=0 (monitor)
		bset	Flags,	#RB5State		; else Drum Kit
;					
		btsc	PORTB,	#RB6			; skip if RB6=0 (disable Midi Clocks)
		bset	Flags,	#RB6State		; else send Midi Clocks
	.else
		.if JP2==1				
			bset	Flags,	#RB5State		; Drum Kit, else monitor
		.endif
		.if	JP3==1
			bset	Flags,	#RB6State		; send Midi Clocks, else disable Midi Clocks
		.endif		
	.endif
;***************************************		
init_UART1:
 		mov		#0x8008,	W0			; UARTEN=1, x, USIDL=0, IREN=0,
										; RTSMD=0, x, UEN1=0, UEN0=0,
										; WAKE=0, LPBACK=0, ABAUD=0, RXINV=0,
										; BRGH=1, PDSEL1=0, PDSEL0=0, STSEL=0
;
 	.if InvertUART1==1
 		bset	W0,	#RXINV
 	.endif
; 	
 		mov		W0,	U1MODE      			; U1 Status	
;
 		mov		#0x8510,	W0			; UTXISEL1=1, UTXINV=0, UTXISEL0=0, x,
 										; UTXBRK=0, UTXEN=1, UTXBF=0, TRMT=1,
										; URXISEL1:=00, ADDEN=0, RIDLE=1,
										; PERR=0, FERR=0, OERR=0, URXDA=0
;
 	.if InvertUART1==1
 		bset	W0,	#UTXINV
 	.endif
; 	
 		mov		W0,	U1STA      			; U1 Status				  
;
		mov 	#32,	W0				; @8MHz 32=31250 baud
		mov		W0,	U1BRG
;
		bclr	IFS0,	#U1RXIF			; clear any interrupt
;***************************************						   
init_UART2:
 		mov		#0x8008,	W0			; UARTEN=1, x, USIDL=0, IREN=0,
										; RTSMD=0, x, UEN1=0, UEN0=0,
										; WAKE=0, LPBACK=0, ABAUD=0, RXINV=0,
										; BRGH=1, PDSEL1=0, PDSEL0=0, STSEL=0
;
 	.if InvertUART2==1
 		bset	W0,	#RXINV
 	.endif
; 	
 		mov		W0,	U2MODE      			; U1 Status	
;
 		mov		#0x0510,	W0			; UTXISEL1=0, UTXINV=0, UTXISEL0=0, x,
 										; UTXBRK=0, UTXEN=1, UTXBF=0, TRMT=1,
										; URXISEL1:=00, ADDEN=0, RIDLE=1,
										; PERR=0, FERR=0, OERR=0, URXDA=0
;
 	.if InvertUART2==1
 		bset	W0,	#UTXINV
 	.endif
; 	
 		mov		W0,	U2STA      			; U2 Status				  
;
		mov 	#8,	W0					; @8MHz 16=115000 baud
		mov		W0,	U2BRG
;
		bclr	IFS1,	#U2RXIF			; clear any interrupt
;***************************************
init_Timer1:							; LCD timer	
		mov		#0x8020,	W0			; TON=0, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=10, x, TSYNC=0, TCS=0, x
		mov		W0,		T1CON			; tick=4MHz/64=16uS
;
		mov		#0x0006,	W0	
		mov		W0,		PR1				; 16*0x0006=96mS
;
		bclr	IFS0,	#T1IF			; clear any interrupt			
;***************************************
init_Timer2:							; "Active Sense" Midi message timer
		mov		#0x8030,	W0			; TON=1, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=11, x, T32=0, TCS=0, x
		mov		W0,		T2CON			; tick=4MHz/256=64uS
;
		mov		#0x124F,	W0	
		mov		W0,		PR2				; 64*0x124F=299mS
;
		bclr	IFS0,	#T2IF			; clear any interrupt
;***************************************
init_Timer3:							; KeyPress timer
		mov		#0x8020,	W0			; TON=1, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=10, x, T32=0, TCS=0, x
		mov		W0,		T3CON			; tick=4MHz/64=16uS
;
		mov		#625,	W0					
		mov		W0,		PR3				; 16*625=10mS		
;
		bclr	IFS0,	#T3IF			; clear any interrupt		
;***************************************
init_Timer4:							; "Midi Clock" Midi message timer
		mov		#0x8030,	W0			; TON=1, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=11, x, T32=0, TCS=0, x
		mov		W0,		T4CON			; tick=4MHz/256=64uS
;
		bclr	IFS1,	#T4IF			; clear any interrupt
;***************************************
init_Timer5:							; KeyPress timer
		mov		#0x8020,	W0			; TON=1, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=10, x, T32=0, TCS=0, x
		mov		W0,		T5CON			; tick=4MHz/64=16uS
;
		mov		#63,	W0					
		mov		W0,		PR5				; 16*63=1mS		
;
		bclr	IFS1,	#T5IF			; clear any interrupt		
;***************************************
init_A_D:
	.if	TriggerType==0					; Analog 
		mov		#0x0270,	W0			; ADON=0, x=0, ADSIDL=0, DMABM=0,
										; DMAEN=0, MODE12=0, FORM1:0=10,  
										; SSRC3:0=0111,  
										; x=0, ASAM=0, SAMP=0, DONE=0
		mov		W0,		AD1CON1			; A/D Control Register 1
;												
		mov		#0x041C,	W0			; PVCFG1:0=00, NVCFG0=0, OFFCAL=0,
										; BUFREGEN=0, CSCNA=1, x=0, x=0,
										; BUFS=0, SMPI4=00111, BUFM=0, ALTS=0
		mov		W0,		AD1CON2			; A/D Control Register 2
;		
		mov		#0x0101,	W0			; ADRC=0, EXTSAM=0, PUMPEN=0, SAMC4=11111,
										; ADCS7=00011111
		mov		W0,		AD1CON3			; A/D Control Register 3
;
		mov		#0x7E03,	W0			; CHH15=0, CHH14=1, CHH13=1, CHH12=1,
										; CHH11=1, CHH10=1, CHH9=1, CHH8=0,
										; CHH7=0, CHH6=0, CHH5=0, CHH4=0,
										; CHH3=0, CHH2=0, CHH1=1, CHH0=1,
		mov		W0,		AD1CSSL			; A/D Scan Select Register (Low Word)
;
		bclr	IFS0,	#AD1IF			; clear any interrupts
		bset	AD1CON1,	#ASAM		; enable autostart
		bset	AD1CON1,	#ADON		; enable A/D		
	.endif										
;												
;***************************************
init_Interrupts:
		mov		PORTA,	W0				; dummy read
		mov		PORTB,	W1				; dummy read
		bclr	IFS1,	#CNIF			; clear changes
;		
		btsc	Flags,	#RB5State		; skip next if JP2 is ON, I'm a monitor
		goto	init_DrumKit			; else I'm a Drum Kit
;
init_Monitor:
		mov		#0x0800,	W0			; NVMIE=0, x, AD1IE=0, U1TXIE=0,
										; U1RXIE=1, SPI1IE=0, SPF1IE=0, T3IE=0,
										; T2IE=0, OC2IE=0, IC2IE=0, x,
										; T1IE=0, OC1IE=0, IC1IE=0, INT0IE=0
		mov		W0,		IEC0
;
		mov		#0x8000,	W0			; U2TXIE=1, U2RXIE=0, INT2IE=0, T5IE=0,
										; T4IE=0, x, OC3IE=0, x,
										; x, x, x, INT1IE=0,
										; CNIE=0, CMIE=0, MI2C1IE=0, SI2C1IE=0
		mov		W0,		IEC1
;
		goto	init_Interrupts_Exit
;
init_DrumKit:
	.if	TriggerType==0					; Analog (ADC)
		mov		#0x3080,	W0			; NVMIE=0, x, AD1IE=1, U1TXIE=1,
										; U1RXIE=0, SPI1IE=0, SPF1IE=0, T3IE=0,
										; T2IE=1, OC2IE=0, IC2IE=0, x,
										; T1IE=0, OC1IE=0, IC1IE=0, INT0IE=0	
	.else								; Digital (no ADC)
		mov		#0x1080,	W0			; NVMIE=0, x, AD1IE=0, U1TXIE=1,
										; U1RXIE=0, SPI1IE=0, SPF1IE=0, T3IE=0,
										; T2IE=1, OC2IE=0, IC2IE=0, x,
										; T1IE=0, OC1IE=0, IC1IE=0, INT0IE=0
	.endif										
		mov		W0,		IEC0
;
	.if	TriggerType==0					; Analog (no CN)
		mov		#0xD800,	W0			; U2TXIE=1, U2RXIE=1, INT2IE=0, T5IE=1,
										; T4IE=1, x, OC3IE=0, x,
										; x, x, x, INT1IE=0,
										; CNIE=0, CMIE=0, MI2C1IE=0, SI2C1IE=0	
	.else								; Digital (CN)
		mov		#0xC808,	W0			; U2TXIE=1, U2RXIE=1, INT2IE=0, T5IE=0,
										; T4IE=1, x, OC3IE=0, x,
										; x, x, x, INT1IE=0,
										; CNIE=1, CMIE=0, MI2C1IE=0, SI2C1IE=0
	.endif
		mov		W0,		IEC1
;		
		goto 	init_Interrupts_Exit
;
init_Interrupts_Exit:
;***************************************		
init_LCD:
		bclr	LATA,	#LED					; force display backlight ON
		call	InitializeLCD
;
		
;***************************************		
init_SignOn:
		btsc	Flags,	#RB5State		; skip next if JP2 is ON, I'm a monitor
		goto	init_SignOn_DrumKit
;
init_SignOn_Monitor:
		mov		#tblpage(SignOn1), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn1), W1	; Make a pointer to table 
		call	SendMessage
;
		goto	init_SignOn_Continue
;				
init_SignOn_DrumKit:		
		mov		#tblpage(SignOn6), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn6), W1	; Make a pointer to table 
		call	SendMessage		
;
init_SignOn_Continue:
		call	SendCRLF
;		
		mov		#tblpage(SignOn2), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn2), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;		
		mov		#tblpage(SignOn3), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn3), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF		
;
		btsc	Flags,	#RB5State		; skip next if JP2 is ON, I'm a monitor
		goto	init_SignOn_DrumKitAgain		
;
init_SignOn_MonitorAgain:				
		mov		#tblpage(SignOn4), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn4), W1	; Make a pointer to table 
		call	SendMessage
;
		goto	init_SignOn_ContinueAgain
;
init_SignOn_DrumKitAgain:
		mov		#tblpage(SignOn5), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignOn5), W1	; Make a pointer to table 
		call	SendMessage		
;
init_SignOn_ContinueAgain:		
		call	SendCRLF
;***************************************
		goto	main
;}
;***************************************;***************************************
;
;	Subroutine: WriteEEPROM	-	 (word)  
;
;	Requires
;		W0	=	Pointer to EEPROM latch
;		W4	=	Pointer to parameters
;	Changes
;		NVMCON,	#WREN
;		NVMCON,	#NVMOP2
;		NVMKEY
;		NVMCON, #WR	
;	Calls
;		none
;
;***************************************;***************************************
;{
WriteEEPROM:
		push	W0
		bset	NVMCON,	#WREN			; enable writing
		bset	NVMCON,	#NVMOP2			; writing 1 word
		disi 	#5						; Disable Interrupts For 5 instructions
		mov 	#0x55,	W0 				; Issue Unlock Sequence
		mov 	W0, 	NVMKEY 
		mov 	#0xAA, 	W0
		mov 	W0, 	NVMKEY
		bset 	NVMCON, #WR				; Perform Write/Erase operations
;
WriteEEPROM_Wait:
		btsc	NVMCON, #WR				; skip next when done
		goto	WriteEEPROM_Wait
;
		pop		W0		
		return
;}
;***************************************;***************************************
;
;	Subroutine: WriteParameters	-	Write Parameters to EEPROM 
;
;	Requires
;		W0	=	which Pad Parameters
;	Changes
;		W0
;		W1
;		W4
;	Calls
;		WriteEEPROM
;
;***************************************;***************************************
;{
WriteParameters:
		rlnc	W0,		W0				; times 2
		rlnc	W0,		W0				; times 2
		mov		#P1,	W4				; P1 is base address
		add.b	W0,		W4,		W4		; add in offset
;	
		mov		#tblpage(MyEEData),	W1
		mov		W1,		_TBLPAG			; Set up a pointer to the EEPROM
		mov		#tbloffset(MyEEData), W1		
		add		W0,		W1,		W0		
		tblwtl	[W4++],	[W0++]			; move to eeprom latch
		call	WriteEEPROM
;						
		tblwtl	[W4++],	[W0++]			; move to eeprom latch
		call	WriteEEPROM	
		return																				     
;}
;***************************************;***************************************
;
;	Main Code Section 
;
;***************************************;***************************************
;{
main:
;***************************************
main_Chk4Button:
		btss	Flags,	#LCDFound		; skip next if LCD is installed
		goto	main_Chk4Button_Exit
;
		btss	IFS0,	#T3IF			; skip next if Timer3 overflow
		goto	main_Chk4Button_Exit
;
		clr		TMR3
		bclr	IFS0,	#T3IF			; clear Timer3 overflow		
		call	Chk4Button
;		
		goto	main_Chk4Button_Exit
;		
main_Chk4Button_Exit:		
;***************************************
main_Mode:		
		btsc	Flags,	#RB5State		; skip next if JP2 is ON, I'm a monitor
		goto	main_DrumKit			; else I'm a Drum Kit
;
main_Monitor:
		call	Monitor
;		
		goto	main_Mode_Exit
;
main_DrumKit:
		call	DrumKit
;		
		goto 	main_Mode_Exit
;
main_Mode_Exit:		
;***************************************		
main_Exit:
		goto	main		
;}
;***************************************;***************************************
;
;	Subroutine: Monitor 	-	Monitor Operation 
;
;	Requires
;		none
;	Changes
;		Flags,	#U1RXCharacterReady
;	Calls
;		U1RXBuffer_Remove
;		call	MidiData
;
;***************************************;***************************************
;{
Monitor:
;***************************************
Monitor_chkU1RX:
		call	U1RXBuffer_Remove
;
		btss	Flags,	#U1RXCharacterReady	; skip next if character available
		goto	Monitor_Exit
;
		bclr	Flags,	#U1RXCharacterReady
		mov.b	U1RXCharacter,	WREG
		call	MidiData
;***************************************		
Monitor_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: DrumKit	-	Drum Kit Operation 
;
;	Requires
;		none
;	Changes
;		W0
;		MyPad
;		MyValue
;		TriggerFlags,	#0
;		TriggerFlags,	#1
;		TriggerFlags,	#2
;		TriggerFlags,	#3
;		TriggerFlags,	#4
;		TriggerFlags,	#5
;		TriggerFlags,	#6
;		TriggerFlags,	#7
;	Calls
;		MyChangeVelocitySetting
;		TriggerPad
;		UpdateLCD
;
;***************************************;***************************************
;{
DrumKit:
;***************************************
DrumKit_chk4Trigger:
;
DrumKit_chk4Trigger1:
		btss	TriggerFlags,	#0		; skip next if 1 is Triggered
		goto	DrumKit_chk4Trigger1_Exit
;
DrumKit_Trigger1:
		mov		#1,		W0
		mov.b	WREG,	MyPad		
		mov.b	In1Max,	WREG	
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#0		; Clear Trigger 1		
DrumKit_chk4Trigger1_Exit:
;
DrumKit_chk4Trigger2:
		btss	TriggerFlags,	#1		; skip next if 2 is Triggered
		goto	DrumKit_chk4Trigger2_Exit
;
DrumKit_Trigger2:
		mov		#2,		W0
		mov.b	WREG,	MyPad		
		mov.b	In2Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;		
		bclr	TriggerFlags,	#1		; Clear Trigger 2
DrumKit_chk4Trigger2_Exit:
;
DrumKit_chk4Trigger3:
		btss	TriggerFlags,	#2		; skip next if 3 is Triggered
		goto	DrumKit_chk4Trigger3_Exit
;
DrumKit_Trigger3:
		mov		#3,		W0
		mov.b	WREG,	MyPad		
		mov.b	In3Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#2		; Clear Trigger 3
DrumKit_chk4Trigger3_Exit:
;
DrumKit_chk4Trigger4:
		btss	TriggerFlags,	#3		; skip next if 4 is Triggered
		goto	DrumKit_chk4Trigger4_Exit
;
DrumKit_Trigger4:
		mov		#4,		W0
		mov.b	WREG,	MyPad		
		mov.b	In4Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#3		; Clear Trigger 4
DrumKit_chk4Trigger4_Exit:
;
DrumKit_chk4Trigger5:
		btss	TriggerFlags,	#4		; skip next if 5 is Triggered
		goto	DrumKit_chk4Trigger5_Exit
;
DrumKit_Trigger5:
		mov		#5,		W0
		mov.b	WREG,	MyPad		
		mov.b	In5Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD

		bclr	TriggerFlags,	#4		; Clear Trigger 5
DrumKit_chk4Trigger5_Exit:
;
DrumKit_chk4Trigger6:
		btss	TriggerFlags,	#5		; skip next if 6 is Triggered
		goto	DrumKit_chk4Trigger6_Exit
;
DrumKit_Trigger6:
		mov		#6,		W0
		mov.b	WREG,	MyPad		
		mov.b	In6Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#5		; Clear Trigger 6
DrumKit_chk4Trigger6_Exit:
;
DrumKit_chk4Trigger7:
		btss	TriggerFlags,	#6		; skip next if 7 is Triggered
		goto	DrumKit_chk4Trigger7_Exit
;
DrumKit_Trigger7:
		mov		#7,		W0
		mov.b	WREG,	MyPad		
		mov.b	In7Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#6		; Clear Trigger 7
DrumKit_chk4Trigger7_Exit:
;
DrumKit_chk4Trigger8:
		btss	TriggerFlags,	#7		; skip next if 8 is Triggered
		goto	DrumKit_chk4Trigger8_Exit
;
DrumKit_Trigger8:
		mov		#8,		W0
		mov.b	WREG,	MyPad		
		mov.b	In8Max,	WREG		
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;		
		call	TriggerPad
;
		call	UpdateLCD
;
		bclr	TriggerFlags,	#7		; Clear Trigger 8
DrumKit_chk4Trigger8_Exit:
;				
DrumKit_chk4Trigger_Exit:
;***************************************
DrumKit_chkU2RX:		
		call	U2RXBuffer_Remove
;
		btss	Flags,	#U2RXCharacterReady	; skip next if character available
		goto	DrumKit_chkU2RX_Exit
;
		bclr	Flags,	#U2RXCharacterReady
		mov.b	U2RXCharacter,	WREG
		call	UserRequest
;		
DrumKit_chkU2RX_Exit:					
;***************************************
DrumKit_chk4PendingMessage:
;
DrumKit_chk4MidiClockPending:
		btss	Flags,	#MidiClockMessage		; skip next if pending 'Midi Clock'
		goto	DrumKit_chk4ActiveSensePending
;
		cp0.b	MidiClockCount
		bra		NZ,		DrumKit_chk4MidiClockPending_NotYet
;
DrumKit_chk4MidiClockPending_Now:
		bset	LATA,	#LED					; force display backlight OFF
		goto	DrumKit_MidiClockMessage
;
DrumKit_chk4MidiClockPending_NotYet:
		bclr	LATA,	#LED					; force display backlight ON
;
DrumKit_MidiClockMessage:
		inc.b	MidiClockCount
		mov		#24,	W1
		mov.b	MidiClockCount,	WREG		
		cpsne.b	W0,		W1
		clr.b	MidiClockCount	
		bclr	Flags,	#MidiClockMessage
		mov.b	#0xF8,	W0
		call	SendMidiCharacter
;
		mov		#tblpage(MessageF8), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF8), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
DrumKit_chk4MidiClockPending_Exit:					
;
DrumKit_chk4ActiveSensePending:		
		btss	Flags,	#ActiveSenseMessage	; skip next if character available
		goto	DrumKit_chk4PendingMessage_Exit
;
		bclr	Flags,	#ActiveSenseMessage
		mov.b	#0xFE,	W0
		call	SendMidiCharacter
;		
		mov		#tblpage(MessageFE), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFE), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
DrumKit_chk4ActiveSensePending_Exit:
;
DrumKit_chk4PendingMessage_Exit:					
;***************************************
DrumKit_Exit:
        return  
;}
;***************************************;***************************************
;
;	Subroutine: Chk4Button	-	Any Buttons Pushed?
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		W4
;		LATB,	#R_WR
;		TRISB
;		TEMP
;		MyMode
;		MyBeats
;		PR4
;		MyInstrument
;		MyVelocity
;		MyNote
;		MyChannel
;		MyValue
;		MyPad
;	Calls
;		TriggerPad
;		UpdateLCD
;		MyChangeInstrumentSetting
;		MyChangeVelocitySetting
;		MyChangeNoteSetting
;		MyChangeChannelSetting
;
;***************************************;***************************************
;{        
Chk4Button:
		bset	LATB,	#R_WR			; force the LCD to let go of the bus
		mov		#0x0F00,W2				; get 0x0F00 into W2
		mov		TRISB,	W3				; get TRISB into W3
		ior		W2,	W3,	W3				; make B8:11 of W3 inputs
		mov		W3,	TRISB				; replace TRISB
		nop								; make sure
		nop								; the bus
		nop								; has
		nop								; settled
		mov		PORTB,	W2				; get PORTB data
		mov		#0x0F00,	W3
		and		W2,		W3,		W2
		mov		W2,	TEMP				; save W10 temporarily		
;
		cp		W2,	 	W3				; Z (no buttons pushed)
		bra		Z,	Chk4Button_Exit 
;
Chk4Button_8:
		btss	W2,		#8			; skip next if D4 is pressed
		goto	Chk4Button_Mode
;
		clr.b	Key8
;
Chk4Button_9:				
		btss	W2,		#9			; skip next if D5 is pressed
		goto	Chk4Button_Enter		
;
		clr.b	Key9
;
Chk4Button_10:				
		btss	W2,		#10			; skip next if D6 is pressed
		goto	Chk4Button_Inc
;
		clr.b	Key10
;		
Chk4Button_11:				
		btss	W2,		#11			; skip next if D7 is pressed
		goto	Chk4Button_Dec
;
		clr.b	Key11		
		goto	Chk4Button_Exit						
;
Chk4Button_Mode:
		inc.b	Key8
		mov		#KeyCount,	W1
		mov.b	Key8,		WREG
		cpsgt.b	W0,		W1
		goto	Chk4Button_Mode_Exit
;		
		clr.b	Key8
		inc.b	MyMode
		mov		#7,		W1
		mov.b	MyMode,	WREG
		cpsne.b	W0,		W1			; skip next if MyMode<>7
		clr.b	MyMode
		call	UpdateLCD		
;
Chk4Button_Mode_Exit:		
		goto	Chk4Button_9
;
Chk4Button_Enter:
		inc.b	Key9
		mov		#KeyCount,	W1
		mov.b	Key9,		WREG
		cpsgt.b	W0,		W1
		goto	Chk4Button_Enter_Exit
;
		clr.b	Key9
		mov.b	MyMode,	WREG		
		mov.b	#0,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>0
		goto	Chk4Button_Enter_Mode0
;
		mov.b	#1,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>1
		goto	Chk4Button_Enter_Mode1
;
		mov.b	#2,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>2
		goto	Chk4Button_Enter_Mode2
;
		mov.b	#3,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>3
		goto	Chk4Button_Enter_Mode3
;
		mov.b	#4,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>4
		goto	Chk4Button_Enter_Mode4
;
		mov.b	#5,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>5
		goto	Chk4Button_Enter_Mode5
;
Chk4Button_Enter_Mode7:				
		clr		W0
		mov.b	MyPad,	WREG
		call	WriteParameters
;
		mov		#tblpage(SavedSettings), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SavedSettings), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	MyPad,	WREG
		call	SendValue
;
		call	SendCRLF
;
		mov		#4,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		mov		#tblpage(SavedSettings),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SavedSettings),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
		mov 	#62500,	W0				; number of 16uS delays (1S)
		mov		W0,		PR1
		call	Delay   				; go wait
;
		mov		#4,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		mov		#tblpage(SignOn5),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn5),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
		goto	Chk4Button_Enter_Exit
;		
Chk4Button_Enter_Mode5:
		nop								; BPM do nothing
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Mode4:
		call	TriggerPad				; Pad, trigger it
;		
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Mode3:
		call	TriggerPad				; Pad, trigger it
;
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Mode2:
		call	TriggerPad				; Pad, trigger it
;
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Mode1:
		call	TriggerPad				; Pad, trigger it
;
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Mode0:
		call	TriggerPad				; Pad, trigger it
;		
		goto	Chk4Button_Enter_Exit
;
Chk4Button_Enter_Exit:
		goto	Chk4Button_10
;
Chk4Button_Inc:
		inc.b	Key10
		mov		#KeyCount,	W1
		mov.b	Key10,		WREG
		cpsgt.b	W0,		W1
		goto	Chk4Button_Inc_Exit
;
		clr.b	Key10
		mov.b	MyMode,	WREG		
		mov.b	#0,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>0
		goto	Chk4Button_Inc_Mode0
;
		mov.b	#1,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>1
		goto	Chk4Button_Inc_Mode1
;
		mov.b	#2,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>2
		goto	Chk4Button_Inc_Mode2
;
		mov.b	#3,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>3
		goto	Chk4Button_Inc_Mode3
;
		mov.b	#4,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>4
		goto	Chk4Button_Inc_Mode4
;
		mov.b	#5,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>5
		goto	Chk4Button_Inc_Mode5
;
Chk4Button_Inc_Mode6:
		nop								; save verify
		goto	Chk4Button_Inc_Exit				
;
Chk4Button_Inc_Mode5:
		inc.b	MyBeats
		mov		#181,	W1
		mov.b	MyBeats,	WREG
		cpseq.b	W0,		W1				; skip next if MyBeats=181
		goto	Chk4Button_Inc_Mode5_Cont
;
		mov		#30,	W0
		mov.b	WREG,	MyBeats
;
Chk4Button_Inc_Mode5_Cont:
		clr		W2
		mov.b	W0,		W2
		mov		#39360,	W0
		repeat	#17
		div.uw	W0,		W2
		mov		W0,		PR4
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit
;
Chk4Button_Inc_Mode4:
		inc.b	MyInstrument
		mov		#128,	W1
		mov.b	MyInstrument,	WREG
		cpseq.b	W0,		W1				; skip next if MyInstrument=128
		goto	Chk4Button_Inc_Mode4_Cont
;
		mov		#0,		W0
		mov.b	WREG,	MyInstrument
;
Chk4Button_Inc_Mode4_Cont:
		mov.b	WREG,	MyValue
		call	ChangeInstrumentSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit
;
Chk4Button_Inc_Mode3:
		inc.b	MyVelocity
		mov		#128,	W1
		mov.b	MyVelocity,	WREG
		cpseq.b	W0,		W1				; skip next if MyVelocity=128
		goto	Chk4Button_Inc_Mode3_Cont
;
		mov		#0,		W0
		mov.b	WREG,	MyVelocity
;
Chk4Button_Inc_Mode3_Cont:
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit
;
Chk4Button_Inc_Mode2:
		inc.b	MyNote
		mov		#128,	W1
		mov.b	MyNote,	WREG
		cpseq.b	W0,		W1				; skip next if MyNote=128
		goto	Chk4Button_Inc_Mode2_Cont
;
		mov		#0,		W0
		mov.b	WREG,	MyNote
;
Chk4Button_Inc_Mode2_Cont:
		mov.b	WREG,	MyValue
		call	ChangeNoteSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit
;
Chk4Button_Inc_Mode1:
		inc.b	MyChannel
		mov		#17,	W1
		mov.b	MyChannel,	WREG
		cpseq.b	W0,		W1				; skip next if MyChannel=17
		goto	Chk4Button_Inc_Mode1_Cont
;
		mov		#1,		W0
		mov.b	WREG,	MyChannel
;
Chk4Button_Inc_Mode1_Cont:
		mov.b	WREG,	MyValue
		call	ChangeChannelSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit							; Channel do nothing
;
Chk4Button_Inc_Mode0:
		inc.b	MyPad
		mov		#9,	W1
		mov.b	MyPad,	WREG
		cpseq.b	W0,		W1				; skip next if MyPad=9
		goto	Chk4Button_Inc_Mode0_Cont
;
		mov		#1,		W0
		mov.b	WREG,	MyPad
;
Chk4Button_Inc_Mode0_Cont:
		call	UpdateLCD
;						
		goto	Chk4Button_Inc_Exit
;
Chk4Button_Inc_Exit:
		goto	Chk4Button_11
;		
Chk4Button_Dec:
		inc.b	Key11
		mov		#KeyCount,	W1
		mov.b	Key11,		WREG
		cpsgt.b	W0,		W1
		goto	Chk4Button_Dec_Exit
;
		clr.b	Key11
		mov.b	MyMode,	WREG		
		mov.b	#0,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>0
		goto	Chk4Button_Dec_Mode0
;
		mov.b	#1,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>1
		goto	Chk4Button_Dec_Mode1
;
		mov.b	#2,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>2
		goto	Chk4Button_Dec_Mode2
;
		mov.b	#3,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>3
		goto	Chk4Button_Dec_Mode3
;
		mov.b	#4,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>4
		goto	Chk4Button_Dec_Mode4
;
		mov.b	#5,		W1
		cpsne.b	W0,		W1			; skip next if MyMode<>5
		goto	Chk4Button_Dec_Mode5
;
Chk4Button_Dec_Mode6:
		nop								; save verify
		goto	Chk4Button_Dec_Exit				
;
Chk4Button_Dec_Mode5:
		dec.b	MyBeats
		mov		#29,	W1
		mov.b	MyBeats,	WREG
		cpseq.b	W0,		W1				; skip next if MyBeats=29
		goto	Chk4Button_Dec_Mode5_Cont
;
		mov		#180,	W0
		mov.b	WREG,	MyBeats
;
Chk4Button_Dec_Mode5_Cont:
		clr		W2
		mov.b	W0,		W2
		mov		#39360,	W0
		repeat	#17
		div.uw	W0,		W2
		mov		W0,		PR4
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Mode4:
		dec.b	MyInstrument
		mov		#255,	W1
		mov.b	MyInstrument,	WREG
		cpseq.b	W0,		W1				; skip next if MyInstrument=255
		goto	Chk4Button_Dec_Mode4_Cont
;
		mov		#127,	W0
		mov.b	WREG,	MyInstrument
;
Chk4Button_Dec_Mode4_Cont:
		mov.b	WREG,	MyValue
		call	ChangeInstrumentSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Mode3:
		dec.b	MyVelocity
		mov		#255,	W1
		mov.b	MyVelocity,	WREG
		cpseq.b	W0,		W1				; skip next if MyVelocity=255
		goto	Chk4Button_Dec_Mode3_Cont
;
		mov		#127,	W0
		mov.b	WREG,	MyVelocity
;
Chk4Button_Dec_Mode3_Cont:
		mov.b	WREG,	MyValue
		call	ChangeVelocitySetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Mode2:
		dec.b	MyNote
		mov		#255,	W1
		mov.b	MyNote,	WREG
		cpseq.b	W0,		W1				; skip next if MyNote=255
		goto	Chk4Button_Dec_Mode2_Cont
;
		mov		#127,	W0
		mov.b	WREG,	MyNote
;
Chk4Button_Dec_Mode2_Cont:
		mov.b	WREG,	MyValue
		call	ChangeNoteSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Mode1:
		dec.b	MyChannel
		mov		#0,	W1
		mov.b	MyChannel,	WREG
		cpseq.b	W0,		W1				; skip next if MyChannel=0
		goto	Chk4Button_Dec_Mode1_Cont
;
		mov		#16,	W0
		mov.b	WREG,	MyChannel
;
Chk4Button_Dec_Mode1_Cont:
		mov.b	WREG,	MyValue
		call	ChangeChannelSetting
;
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Mode0:
		dec.b	MyPad
		mov		#0,	W1
		mov.b	MyPad,	WREG
		cpseq.b	W0,		W1				; skip next if MyPad=0
		goto	Chk4Button_Dec_Mode0_Cont
;
		mov		#8,	W0
		mov.b	WREG,	MyPad
;
Chk4Button_Dec_Mode0_Cont:
		call	UpdateLCD
;						
		goto	Chk4Button_Dec_Exit
;
Chk4Button_Dec_Exit:
		goto	Chk4Button_Exit																			
;
Chk4Button_Exit:
		return		                 			
;}
;***************************************;***************************************
;
;	Subroutine: Get Midi Data and do something
;
;	Requires
;		W0	=	Status byte from U2RXBuffer
;	Changes
;		W0
;		W1
;		W2
;		W3
;		MessageNumber
;		ChannelNumber
;		MessageDataCount
;		SongNumber
;		MidiBeatNumber
;		TimeCodeNumber
;		TransposeNumber
;		PressureNumber
;		ProgramNumber
;		ValueNumber
;		ControllerNumber
;		NoteNumber
;		VelocityNumber
;	Calls
;		SystemMessage
;		SendValue
;		SendCharacter
;		OutputMessageF3
;		OutputMessageF2
;		OutputMessageF1
;		OutputMessageE
;		OutputMessageD
;		OutputMessageA
;		OutputMessageC
;		OutputMessageB
;		OutputMessage9
;		OutputMessage8
;
;***************************************;***************************************
;{
MidiData:
		btss	W0,		#7				; skip next if not a status byte
		goto	MidiData_NonStatus
;
MidiData_Status:
		btss	W0,		#6
		goto	MidiData_Status_10xx
;
MidiData_Status_11xx:
		btss	W0,		#5
		goto	MidiData_Status_110x
;
MidiData_Status_111x:
		btss	W0,		#4
		goto	MidiData_Status_1110
;
;***************************************
;
MidiData_Status_1111:					; System
		call	SystemMessage
;
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_1110:					; Pitch Wheel
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,W0
		and.b	ChannelNumber				
		mov		#2,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_110x:
		btss	W0,		#4
		goto	MidiData_Status_1100
;
;***************************************
;
MidiData_Status_1101:					; Channel Pressure
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber				
		mov		#1,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_1100:					; Program Patch
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber				
		mov		#1,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_10xx:
		btss	W0,		#5
		goto	MidiData_Status_100x
;
MidiData_Status_101x:
		btss	W0,		#4
		goto	MidiData_Status_1010
;
;***************************************
;
MidiData_Status_1011:					; Control Change
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber				
		mov		#2,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_1010:					; After Touch
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber				
		mov		#2,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_100x:
		btss	W0,		#4
		goto	MidiData_Status_1000
;
;***************************************
;
MidiData_Status_1001:					; Note On
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber				
		mov		#2,		W0
		mov.b	WREG,	MessageDataCount
		goto	MidiData_Exit
;
;***************************************
;
MidiData_Status_1000:					; Note Off
		mov.b	WREG,	MessageNumber
		mov.b	WREG,	ChannelNumber
		mov.b	#0xF0,	W0
		and.b	MessageNumber
		mov.b	#0x0F,	W0
		and.b	ChannelNumber		
		mov		#2,		W0
		mov.b	WREG,	MessageDataCount
;
		goto	MidiData_Exit		
;
;***************************************
;
MidiData_NonStatus:
		mov.b	WREG,	TempW
		mov.b	#0,	W2
		mov.b	MessageDataCount,	WREG
		mov		W0,	W3
		cpsne.b	W2,	W3
		goto	MidiData_Exit	
;
		mov.b	#1,	W2	
		cpseq.b	W2,	W3
		goto	MidiData_Cont
;
MidiData_NonStatus_1:
		mov		#0x80,	W2
		mov		MessageNumber,	W3
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_Velocity8
;		
		mov		#0x90,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_Velocity9
;		
		mov		#0xA0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_PressureA
;		
		mov		#0xB0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_Value
;		
		mov		#0xC0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_Program
;		
		mov		#0xD0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_PressureD
;		
		mov		#0xE0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_Transpose
;		
		mov		#0xF1,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_TimeCode
;		
		mov		#0xF2,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_MidiBeat		
;		
		mov		#0xF3,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_1_SongNumber
;
		goto	MidiData_Exit		
;
MidiData_NonStatus_1_SongNumber:
		mov.b	TempW,	WREG
		inc.b	W0,		W0
		mov.b	WREG,	SongNumber
		clr.b	MessageDataCount
		call	OutputMessageF3		
		goto	MidiData_Exit
;
MidiData_NonStatus_1_MidiBeat:
		mov.b	TempW,	WREG
		mov		MidiBeatNumber,	W1
		and.b	W0,	W1,	W1
		mov		W1,	MidiBeatNumber
		clr.b	MessageDataCount
		call	OutputMessageF2		
		goto	MidiData_Exit
;
MidiData_NonStatus_1_TimeCode:
		mov.b	TempW,	WREG
		mov.b	WREG,	TimeCodeNumber
		clr.b	MessageDataCount
		call	OutputMessageF1				
		goto	MidiData_Exit
;
MidiData_NonStatus_1_Transpose:
		mov.b	TempW,	WREG
		mov		TransposeNumber,	W1
		and.b	W0,	W1,	W1
		mov		W1,	TransposeNumber
		mov.b	#2,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessageE				
		goto	MidiData_Exit
;
MidiData_NonStatus_1_PressureD:
		mov.b	TempW,	WREG
		mov.b	WREG,	PressureNumber
		mov.b	#1,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessageD				
		goto	MidiData_Exit		
;
MidiData_NonStatus_1_PressureA:
		mov.b	TempW,	WREG
		mov.b	WREG,	PressureNumber
		mov.b	#2,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessageA				
		goto	MidiData_Exit
;
MidiData_NonStatus_1_Program:
		mov.b	TempW,	WREG
		mov.b	WREG,	ProgramNumber
		mov.b	#1,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessageC				
		goto	MidiData_Exit
;
MidiData_NonStatus_1_Value:
		mov.b	TempW,	WREG
		mov.b	WREG,	ValueNumber
		mov.b	#2,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessageB
		goto	MidiData_Exit
;
MidiData_NonStatus_1_Velocity9:
		mov.b	TempW,	WREG
		mov.b	WREG,	VelocityNumber
		mov.b	#2,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessage9		
		goto	MidiData_Exit		
;
MidiData_NonStatus_1_Velocity8:
		mov.b	TempW,	WREG
		mov.b	WREG,	VelocityNumber
		mov.b	#2,		W0
		mov.b	WREG,	MessageDataCount
		call	OutputMessage8		
		goto	MidiData_Exit																													
;
MidiData_Cont:
		mov.b	#2,	W2	
		cpseq.b	W2,	W3
		goto	MidiData_NonStatus_3
;
MidiData_NonStatus_2:
		mov		#0x80,	W2
		mov		MessageNumber,	W3
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Note
;		
		mov		#0x90,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Note
;		
		mov		#0xA0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Note
;		
		mov		#0xB0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Controller
;		
		mov		#0xC0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Program
;		
		mov		#0xD0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Pressure				
;		
		mov		#0xE0,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_Transpose
;		
		mov		#0xF2,	W2
		cp.b	W2,	W3
		bra		z,	MidiData_NonStatus_2_MidiBeat				
;		
		goto	MidiData_Exit
;
MidiData_NonStatus_2_MidiBeat:
		mov.b	TempW,	WREG
		swap.b	W0
		lsr		W0,	W0
		mov		W0,	MidiBeatNumber
		dec.b	MessageDataCount
		goto	MidiData_Exit				
;
MidiData_NonStatus_2_Transpose:
		mov.b	TempW,	WREG
		swap.b	W0
		lsr		W0,	W0
		mov		W0,	TransposeNumber
		dec.b	MessageDataCount		
		goto	MidiData_Exit
;		
MidiData_NonStatus_2_Pressure:
		mov.b	TempW,	WREG
		mov.b	WREG,	PressureNumber
		dec.b	MessageDataCount		
		goto	MidiData_Exit
;		
MidiData_NonStatus_2_Program:
		mov.b	TempW,	WREG
		mov.b	WREG,	ProgramNumber
		dec.b	MessageDataCount		
		goto	MidiData_Exit				
;		
MidiData_NonStatus_2_Controller:
		mov.b	TempW,	WREG
		mov.b	WREG,	ControllerNumber
		dec.b	MessageDataCount		
		goto	MidiData_Exit
;				
MidiData_NonStatus_2_Note:
		mov.b	TempW,	WREG
		mov.b	WREG,	NoteNumber
		dec.b	MessageDataCount				
		goto	MidiData_Exit
;
MidiData_NonStatus_3:
		mov.b	TempW,	WREG
		call	SendValue
;
		mov		#' ',	W0
		call	SendCharacter
;				
		goto	MidiData_Cont
;
MidiData_Exit:				
		return  
;}
;***************************************;***************************************
;
;	Subroutine: SystemMessage	-	Do System Message (Terminal Display)
;
;	Requires
;		W0	=	Status byte from U2RXBuffer
;		zero terminated string (MessageFF)
;		zero terminated string (MessageFE)
;		zero terminated string (MessageFC)
;		zero terminated string (MessageFB)
;		zero terminated string (MessageFA)
;		zero terminated string (MessageF9)
;		zero terminated string (MessageF8)
;		zero terminated string (MessageF7)
;		zero terminated string (MessageF6)
;		zero terminated string (MessageF3)
;		zero terminated string (MessageF2)
;		zero terminated string (MessageF1)
;		zero terminated string (MessageF0)
;	Changes
;		W0
;		W1
;		MessageNumber
;		MessageDataCount
;	Calls
;		SendMessage
;		SendCRLF
;
;***************************************;***************************************
;{
SystemMessage:
		btss	W0,		#3				; skip next if not a status byte
		goto	SystemMessage_11110xxx
		
;
SystemMessage_11111xxx:
		btss	W0,		#2
		goto	SystemMessage_111110xx
;
SystemMessage_111111xx:
		btss	W0,		#1
		goto	SystemMessage_1111110x
;
SystemMessage_1111111x:
		btss	W0,		#0
		goto	SystemMessage_11111110
;
;***************************************
;
SystemMessage_11111111:					; System Reset
		mov		#0xFF,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageFF), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFF), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11111110:					; System Active Sense
		mov		#0xFE,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageFE), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFE), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_1111110x:
		btss	W0,		#0
		goto	SystemMessage_11111100
;
;***************************************
;
SystemMessage_11111101:					; System (none)
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11111100:					; System Midi Stop
		mov		#0xFC,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageFC), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFC), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_111110xx:
		btss	W0,		#1
		goto	SystemMessage_1111100x
;
SystemMessage_1111101x:
		btss	W0,		#0
		goto	SystemMessage_11111010
;
;***************************************
;
SystemMessage_11111011:					; System Midi Continue
		mov		#0xFB,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageFB), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFB), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11111010:					; System Midi Start
		mov		#0xFA,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageFA), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageFA), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_1111100x:
		btss	W0,		#0
		goto	SystemMessage_11111000
;
;***************************************
;
SystemMessage_11111001:					; System Tick
		mov		#0xF9,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageF9), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF9), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11111000:					; System Midi Clock
		mov		#0xF8,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageF8), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF8), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit		
;
;***************************************
;
SystemMessage_11110xxx:
		btss	W0,		#2
		goto	SystemMessage_111100xx
;
SystemMessage_111101xx:
		btss	W0,		#1
		goto	SystemMessage_1111010x
;
SystemMessage_1111011x:
		btss	W0,		#0
		goto	SystemMessage_11110110
;
;***************************************
;
SystemMessage_11110111:					; System  End Of SysEx
		mov		#0xF7,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageF7), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF7), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11110110:					; System Tune Request
		mov		#0xF6,	W0
		mov.b	WREG,		MessageNumber
		mov		#0,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageF6), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF6), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_1111010x:
		btss	W0,		#0
		goto	SystemMessage_11110100
;
;***************************************
;
SystemMessage_11110101:					; System (none)
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11110100:					; System (none)
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_111100xx:
		btss	W0,		#1
		goto	SystemMessage_1111000x
;
SystemMessage_1111001x:
		btss	W0,		#0
		goto	SystemMessage_11110010
;
;***************************************
;
SystemMessage_11110011:					; System Song Select
		mov		#0xF3,	W0
		mov.b	WREG,		MessageNumber
		mov		#1,		W0
		mov.b	WREG,	MessageDataCount
		mov		#tblpage(MessageF3), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF3), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;						
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11110010:					; System Song Pointer
		mov		#0xF2,	W0
		mov.b	WREG,		MessageNumber
		mov		#2,		W0
		mov.b	WREG,		MessageDataCount
		mov		#tblpage(MessageF2), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF2), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_1111000x:
		btss	W0,		#0
		goto	SystemMessage_11110000
;
;***************************************
;
SystemMessage_11110001:					; System MTC Quarter Frame Message
		mov		#0xF1,	W0
		mov.b	WREG,		MessageNumber
		mov		#1,		W0
		mov.b	WREG,		MessageDataCount
		mov		#tblpage(MessageF1), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF1), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
		goto	SystemMessage_Exit
;
;***************************************
;
SystemMessage_11110000:					; System Start Of SysEx
		mov		#0xF0,	W0
		mov.b	WREG,		MessageNumber
		mov		#3,		W0
		mov.b	WREG,		MessageDataCount
		mov		#tblpage(MessageF0), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF0), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
		goto	SystemMessage_Exit		
;
SystemMessage_Exit:				
		return 
;}
;***************************************;***************************************
;
;	Subroutine: Do User Request	-	(Terminal Display)
;
;	Px<cr>	=	set pad number 1-8
;	Cx<cr>	=	set channel number 1-16
;	Vx<cr>	=	set voice (instrumemt) number 0-127
;	Nx<cr>	=	set pad note number 0-127
;	Sx<cr>	=	save pad configuration
;	Tx<cr>	=	trigger pad number 1-8
;	Bx<cr>	=	beats number 30-180
;
;	Requires
;		W0	=	User Character from U2TXBuffer
;		zero terminated string (MyEEData)
;		zero terminated string (SavedSettings)
;		zero terminated string (Beats)
;	Changes
;		W0
;		W1
;		W4
;		TEMPW
;		MyValue
;		MyRequest
;		MyPad
;		MyBeats
;		Flags,	#Request
;	Calls
;		OutputEntryError
;		OutputPadSettings
;		ChangeChannelSetting
;		ChangeNoteSetting
;		ChangeVelocitySetting
;		ChangeInstrumentSetting
;		ChangeBPMSetting
;		WriteEEPROM
;		SendMessage
;		SendValue
;		SendCRLF
;		TriggerPad		
;		TriggeringMessageToUART
;		
;
;***************************************;***************************************
;{
UserRequest:
		mov.b	#'a',	W1
		cpslt	W0,		W1
		bclr	W0,		#5				; force to upper case
;		
		mov.b	#'+',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_Plus
;		
		mov.b	#'-',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_Minus
;		
		mov.b	#'P',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest				
;
		mov.b	#'C',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest
;
		mov.b	#'V',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest
;
		mov.b	#'N',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest
;
		mov.b	#'S',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest
;
		mov.b	#'B',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest		
;
		mov.b	#'T',	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest
;
		mov.b	#'I,	W1
		cpsne.b	W0,		W1
		goto	UserRequest_SaveRequest		
;
		mov.b	#0x0D,	W1
		cpsne.b	W0,		W1
		goto	UserRequest_CR		
;
		mov.b	#'/',	W1
		cpsgt.b	W0,		W1
		goto	UserRequest_Bad
;		
		mov.b	#':',		W1
		cpslt.b	W0,		W1		
		goto	UserRequest_Bad
;
UserRequest_Number:
		mov.b	WREG,		TempW
		clr		W0		
		mov.b	MyValue,	WREG
		mov		#10,	W1
		mul.uu 	W0,		W1,		W2
		mov.b	TempW,	WREG
		and.b	#0xF,	W0
		add.b	W0,		W2,		W0
		mov.b	WREG,	MyValue
		bset	Flags,	#Request
		goto	UserRequest_Exit
;
UserRequest_SaveRequest:
		mov.b	WREG,	MyRequest
		bclr	Flags,	#Request
		clr.b	MyValue
		goto	UserRequest_Exit		
;
UserRequest_CR:
		btss	Flags,	#Request			; skip next if we have a Legal Request
		goto	UserRequest_Bad
;		
		mov.b	MyRequest,	WREG
		mov.b	#'P',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_P
;
		mov.b	MyRequest,	WREG
		mov.b	#'C',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_C
;
		mov.b	MyRequest,	WREG
		mov.b	#'V',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_V
;
		mov.b	MyRequest,	WREG
		mov.b	#'I',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_I		
;
		mov.b	MyRequest,	WREG
		mov.b	#'N',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_N
;
		mov.b	MyRequest,	WREG
		mov.b	#'S',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_S
;
		mov.b	MyRequest,	WREG
		mov.b	#'B',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_B
;
		mov.b	MyRequest,	WREG
		mov.b	#'T',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_T		
;
UserRequest_Bad:
		call	OutputEntryError
;		
UserRequest_Cleanup:
		clr.b	MyRequest
		clr.b	MyValue		
		goto	UserRequest_Exit
;
UserRequest_Plus:
		mov.b	MyRequest,	WREG
;		
		mov.b	#'T',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_T
;		
		mov.b	#'P',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_P
;		
		mov.b	#'C',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_C
;		
		mov.b	#'V',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_V
;		
		mov.b	#'I',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_I		
;		
		mov.b	#'N',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_N	
;		
		mov.b	#'B',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Plus_B										
;			
		goto	UserRequest_Bad
;
UserRequest_Plus_T:
		inc.b	MyValue
		mov.b	MyValue,	WREG
		mov.b	#8,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_TGood
;		
		mov		#1,		W0
		mov.b	WREG,	MyValue
		goto	UserRequest_TGood
;
UserRequest_Plus_P:
		inc.b	MyPad,	WREG
		mov.b	#9,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_PGood
		mov		#1,		W0
		goto	UserRequest_PGood
;
UserRequest_Plus_C:
		inc.b	MyValue,	WREG
		mov.b	#17,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_Plus_CSave
		mov		#1,		W0
;
UserRequest_Plus_CSave:
		mov.b	WREG,	MyValue		
		goto	UserRequest_C
;
UserRequest_Plus_V:
		inc.b	MyValue,	WREG
		mov.b	#128,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_Plus_VSave
		mov		#0,		W0
;
UserRequest_Plus_VSave:
		mov.b	WREG,	MyValue
		goto	UserRequest_V
;
UserRequest_Plus_I:
		inc.b	MyValue,	WREG
		mov.b	#128,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_Plus_ISave
		mov		#0,		W0
;
UserRequest_Plus_ISave:
		mov.b	WREG,	MyValue
		goto	UserRequest_I		
;
UserRequest_Plus_N:
		inc.b	MyValue,	WREG
		mov.b	#128,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_Plus_NSave
		mov		#1,		W0
;
UserRequest_Plus_NSave:
		mov.b	WREG,	MyValue
		goto	UserRequest_N		
;
UserRequest_Plus_B:
		inc.b	MyBeats,	WREG
		mov.b	#181,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_BGood
;		
		mov		#30,		W0
		goto	UserRequest_BGood									
;
UserRequest_Minus:
		mov.b	MyRequest,	WREG
;		
		mov.b	#'T',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_T
;		
		mov.b	#'P',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_P
;		
		mov.b	#'C',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_C
;		
		mov.b	#'V',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_V	
;		
		mov.b	#'I',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_I		
;		
		mov.b	#'N',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_N	
;		
		mov.b	#'B',	W1
		cpsne.b	W0,	W1
		goto	UserRequest_Minus_B											
;			
		goto	UserRequest_Bad
;
UserRequest_Minus_T:
		dec.b	MyValue
		cp0.b	MyValue
		bra		nz,	UserRequest_TGood
;		
		mov		#8,		W0
		mov.b	WREG,	MyValue
		goto	UserRequest_TGood
;
UserRequest_Minus_P:
		dec.b	MyPad,	WREG
		cp0.b	W0
		bra		nz,	UserRequest_PGood
;		
		mov		#8,		W0
		goto	UserRequest_PGood
;
UserRequest_Minus_C:
		dec.b	MyValue,	WREG
		cp0.b	W0
		bra		nz,	UserRequest_Minus_CSave
		mov		#16,	W0
;
UserRequest_Minus_CSave:
		mov.b	WREG,	MyValue
		goto	UserRequest_C
;
UserRequest_Minus_V:
		dec.b	MyValue,	WREG
		cp0.b	W0
		bra		nz,	UserRequest__Minus_VSave
		mov		#127,	W0
;
UserRequest__Minus_VSave:
		mov.b	WREG,	MyValue
		goto	UserRequest_V
;
UserRequest_Minus_I:
		dec.b	MyValue,	WREG
		cp0.b	W0
		bra		nz,	UserRequest__Minus_ISave
		mov		#127,	W0
;
UserRequest__Minus_ISave:
		mov.b	WREG,	MyValue
		goto	UserRequest_I		
;
UserRequest_Minus_N:
		dec.b	MyValue,	WREG
		cp0.b	W0
		bra		nz,	UserRequest_Minus_NSave
		mov		#127,	W0
;
UserRequest_Minus_NSave:
		mov.b	WREG,	MyValue
		goto	UserRequest_N
;
UserRequest_Minus_B:
		inc.b	MyBeats,	WREG
		mov.b	#29,	W1
		cp.b	W0,		W1
		bra		nz,	UserRequest_BGood
;		
		mov		#180,		W0
		goto	UserRequest_BGood											
;
UserRequest_P:							; Request to set Pad
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#9,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_PGood:		
		mov.b	WREG,		MyPad
		call	OutputPadSettings
;		
		goto	UserRequest_Exit	
;
UserRequest_C:							; Request to set Channel
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#17,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_CGood:		
		mov.b	WREG,		MyValue
		call	ChangeChannelSetting
;		
		call	OutputPadSettings
;		
		goto	UserRequest_Exit
;
UserRequest_V:							; Request to set Velocity
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#129,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_VGood:		
		mov.b	WREG,		MyValue
		call	ChangeVelocitySetting
;		
		call	OutputPadSettings
;		
		goto	UserRequest_Exit
;
UserRequest_I:							; Request to set Instrument
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#129,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_IGood:		
		mov.b	WREG,		MyValue
		call	ChangeInstrumentSetting
;		
		call	OutputPadSettings
;		
		goto	UserRequest_Exit		
;
UserRequest_N:							; Request to set Note
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#129,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_NGood:		
		mov.b	WREG,		MyValue
		call	ChangeNoteSetting
;		
		call	OutputPadSettings
;		
		goto	UserRequest_Exit
;
UserRequest_S:							; Request to Save settings
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#9,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_SGood:
		mov.b	WREG,		MyValue
		clr		W0
		dec.b	MyValue,	WREG
		call	WriteParameters	
;
		mov		#tblpage(SavedSettings), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SavedSettings), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	MyValue,	WREG
		call	SendValue
;
		call	SendCRLF				
;		
		goto	UserRequest_Cleanup	
;
UserRequest_T:							; Request to trigger
		mov.b	MyValue,	WREG
		cp0.b	MyValue
		bra		Z,	UserRequest_Bad
		mov.b	#9,	W1
		cpslt	W0,	W1
		goto	UserRequest_Bad
;
UserRequest_TGood:
		mov.b	MyValue,	WREG
		mov.b	WREG,	MyPad
		call	TriggerPad
;		
		call	TriggeringMessageToUART		
;
		goto	UserRequest_Exit
;
UserRequest_B:							; Request a BPM (tempo)
		clr		W0
		mov.b	MyValue,	WREG
		mov		#30,	W1
		cp.b	W0,	W1
		bra		ltu,	UserRequest_Bad
;		
		mov		#180,	W1
		cp.b	W0,	W1
		bra		gtu,	UserRequest_Bad		
;
UserRequest_BGood:
		mov.b	WREG,	MyBeats
		mov		W0,		W2
		mov		#39360,	W0
		repeat	#17
		div.uw	W0,		W2
		mov		W0,		PR4
;		
		mov		#tblpage(Beats), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Beats), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	MyBeats,	WREG
		call	SendValue
;
		call	SendCRLF		
;
		goto	UserRequest_Exit
UserRequest_Exit:
		return		      		     
;}
;***************************************;***************************************
;
;	Subroutine: Add a character to the U1TX Buffer
;
;	Requires
;		U1TXCharacter	-	Character to add
;	Changes
;		U1TXBuffer_Head
;		IEC0,		#U1TXIE
;	Calls
;		none
;
;***************************************;***************************************
;{
U1TXBuffer_Add:
		push.d	W0
;
U1TXBuffer_Add_Loop:		
		disi	#9
		mov		U1TXBuffer_Head,	W1
		inc		W1,	W1
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cpsne	W0,			W1			; skip next if past the U1TXBuffer
;
		mov		#U1TXBuffer,	W1
;				
		mov		U1TXBuffer_Tail,	W0
		cpseq	W0,			W1			; skip next if there is no room
		goto	U1TXBuffer_Add_Room
;
U1TXBuffer_Add_NoRoom:
		nop
		nop
		nop
		nop
		goto	U1TXBuffer_Add_Loop
;
U1TXBuffer_Add_Room:
		disi	#9
		mov		U1TXBuffer_Head,	W1
		mov.b	U1TXCharacter,	WREG
		mov.b	W0,			[W1++]		
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cpseq	W1,			W0			; skip next if past the U1TXBuffer
		goto	U1TXBuffer_Add_Room_NoWrap	
;
U1TXBuffer_Add_Room_Wrap:
		mov		#U1TXBuffer,	W1
;
U1TXBuffer_Add_Room_NoWrap:
		mov		W1,			U1TXBuffer_Head
		bset	IEC0,		#U1TXIE		; Enable U1Transmit Interrupt	
;
U1TXBuffer_Add_Exit:
		pop.d	W0			
		return
;}
;***************************************;***************************************
;
;	Subroutine: Check the U1RX Buffer for characters
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		Flags,	#U1RXEmpty
;	Calls
;		none
;
;***************************************;***************************************
;{
U1RXBuffer_Check:
		mov		U1RXBuffer_Head,	W0
		mov		U1RXBuffer_Tail,	W1
		cpseq	W0,	W1					; skip next if buffer is empty
		goto	U1RXBuffer_Check_NotEmpty
;
U1RXBuffer_Check_Empty:
		bset	Flags,	#U1RXEmpty
		goto	U1RXBuffer_Check_Exit
;
U1RXBuffer_Check_NotEmpty:
		bclr	Flags,	#U1RXEmpty
		goto	U1RXBuffer_Check_Exit
;
U1RXBuffer_Check_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Remove a character from the U1RX Buffer
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		U1RXBuffer_Tail
;		U1RXCharacter
;	Calls
;		U1RXBuffer_Check
;
;***************************************;***************************************
;{
U1RXBuffer_Remove:
		disi	#20
		call	U1RXBuffer_Check
;
		btsc	Flags,		#U1RXEmpty	; skip next if its not Empty
		goto	U1RXBuffer_Remove_Exit
;
		mov		U1RXBuffer_Tail,	W1
		mov.b	[W1++],		W0
		mov.b	WREG,		U1RXCharacter
		mov		#U1RXBuffer+UARTBuffer_Length,	W0
		cpseq	W0,			W1			; skip next if past the Buffer
		goto	U1RXBuffer_Remove_Room_NoWrap
;
U1RXBuffer_Remove_Room_Wrap:
		mov		#U1RXBuffer,	W1
;
U1RXBuffer_Remove_Room_NoWrap:
		mov		W1,	U1RXBuffer_Tail	
		bset	Flags,	#U1RXCharacterReady					
;
U1RXBuffer_Remove_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Add a character to the U2TXBuffer	-	(Terminal Display)
;
;	Requires
;		U2TXCharacter	-	Character to send
;	Changes
;		U2TXBuffer_Head
;	Calls
;		none
;
;***************************************;***************************************
;{
U2TXBuffer_Add:
		push.d	W0
;
U2TXBuffer_Add_Loop:		
		disi	#9
		mov		U2TXBuffer_Head,	W1
		inc		W1,	W1
		mov		#U2TXBuffer+UARTBuffer_Length,	W0
		cpsne	W0,			W1			; skip next if past the U2TXBuffer
;
		mov		#U2TXBuffer,	W1
;				
		mov		U2TXBuffer_Tail,	W0
		cpseq	W0,			W1			; skip next if there is no room
		goto	U2TXBuffer_Add_Room
;
U2TXBuffer_Add_NoRoom:
		nop
		nop
		nop
		nop
		goto	U2TXBuffer_Add_Loop
;
U2TXBuffer_Add_Room:
		disi	#9
		mov		U2TXBuffer_Head,	W1
		mov.b	U2TXCharacter,	WREG
		mov.b	W0,			[W1++]		
		mov		#U2TXBuffer+UARTBuffer_Length,	W0
		cpseq	W1,			W0			; skip next if past the U2TXBuffer
		goto	U2TXBuffer_Add_Room_NoWrap	
;
U2TXBuffer_Add_Room_Wrap:
		mov		#U2TXBuffer,	W1
;
U2TXBuffer_Add_Room_NoWrap:
		mov		W1,			U2TXBuffer_Head
		bset	IEC1,		#U2TXIE		; Enable U2Transmit Interrupt	
;
U2TXBuffer_Add_Exit:
		pop.d	W0			
		return
;}
;***************************************;***************************************
;
;	Subroutine: Check U2RXBuffer for characters (Flag set if empty, else clear)
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		Flags,	#U2RXEmpty
;	Calls
;		none
;
;***************************************;***************************************
;{
U2RXBuffer_Check:
		mov		U2RXBuffer_Head,	W0
		mov		U2RXBuffer_Tail,	W1
		cpseq	W0,	W1					; skip next if buffer is empty
		goto	U2RXBuffer_Check_NotEmpty
;
U2RXBuffer_Check_Empty:
		bset	Flags,	#U2RXEmpty
		goto	U2RXBuffer_Check_Exit
;
U2RXBuffer_Check_NotEmpty:
		bclr	Flags,	#U2RXEmpty
		goto	U2RXBuffer_Check_Exit
;
U2RXBuffer_Check_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Remove a character from U2RXBuffer (U2RXCharacter=Character)
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		U2RXBuffer_Tail
;		U2RXCharacter
;	Calls
;		U2RXBuffer_Check
;
;***************************************;***************************************
;{
U2RXBuffer_Remove:
		disi	#20
		call	U2RXBuffer_Check
;
		btsc	Flags,		#U2RXEmpty	; skip next if its not Empty
		goto	U1RXBuffer_Remove_Exit
;
		mov		U2RXBuffer_Tail,	W1
		mov.b	[W1++],		W0
		mov.b	WREG,		U2RXCharacter
		mov		#U2RXBuffer+UARTBuffer_Length,	W0
		cpseq	W0,			W1			; skip next if past the Buffer
		goto	U2RXBuffer_Remove_Room_NoWrap
;
U2RXBuffer_Remove_Room_Wrap:
		mov		#U2RXBuffer,	W1
;
U2RXBuffer_Remove_Room_NoWrap:
		mov		W1,	U2RXBuffer_Tail	
		bset	Flags,	#U2RXCharacterReady					
;
U2RXBuffer_Remove_Exit:
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Initialization of W0:14 registers to 0x0000
;
;	Requires
;		none
;	Changes
;		W0
;		W14
;	Calls
;		none
;
;***************************************;***************************************
;{
_wreg_init:
        clr 	W0
        mov 	W0,	W14
        repeat 	#12
        mov 	W0, [++W14]
        clr 	W14
        return
;}
;***************************************;***************************************
;
;	Subroutine: Send Triggering Message to UART (Terminal Display)
;
;	Required
;		zero terminated string (Triggering)
;	Changes
;		W0
;	Calls
;		SendMessage
;		OutputPadSettings
;		SendCRLF
;
;***************************************;***************************************
;{
TriggeringMessageToUART:		
		mov		#tblpage(Triggering), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Triggering), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	MyPad,	WREG
		call	OutputPadSettings
;
		call	SendCRLF
;
		return
;}		
;***************************************;***************************************
;
;	Subroutine: Send Midi Note ON and OFF (Midi)
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		none
;	Calls
;		TriggerPad_NoteON
;		TriggerPad_NoteOFF
;
;***************************************;***************************************
;{
TriggerPad:
		call	TriggerPad_NoteON
;
		call	TriggerPad_NoteOFF
;
		return
;
										
;
;***************************************;***************************************
;
;	Subroutine: Send Midi Note ON	(Midi) (Terminal Display OutputMessage9)
;	Format	-	#0x90+channel, note, velocity
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W1
;		W4
;		ChannelNumber
;		NoteNumber
;		VelocityNumber
;		TMR2
;	Calls
;		SendMidiCharacter
;		TriggerPad_NoteOFF
;		OutputMessage9
;
;***************************************;***************************************
;
TriggerPad_NoteON:
		clr		W0
		mov.b	MyPad,	WREG			; get the Pad#
		dec.b	W0,		W0
		rlnc	W0,		W0				; times 2
		rlnc	W0,		W0				; times 2
		mov		#P1,	W4				; P1 is base address
		add.b	W0,		W4,		W4		; add in offset
		mov.b	[W4++],	W1				; channel value at offset
		dec.b	W1,		W1				; minus 1
		mov		W1,		W0		
		mov.b	WREG,	ChannelNumber
		mov.b	WREG,	MyChannel		
;	
		mov.b	#0x90,	W0				; status for Note ON
		add.b	W1,		W0,		W0		; total status including channel
		call	SendMidiCharacter
;
		mov.b	[W4++],	W0				; note value at offset
		mov.b	WREG,	NoteNumber
		mov.b	WREG,	MyNote				
		call	SendMidiCharacter
;
		mov.b	[W4++],	W0				; voice value at offset
		mov.b	[W4++],	W0				; velocity value at offset
		mov.b	WREG,	VelocityNumber
		mov.b	WREG,	MyVelocity		
		call	SendMidiCharacter
;
		call	OutputMessage9
;		
TriggerPad_NoteON_Exit:
		clr		TMR2					; reset the Active Sense Timer
		return		
;
;***************************************;***************************************
;
;	Subroutine: Send Midi Note OFF	(Midi) (Terminal Display OutputMessage8)
;	Format	#0x80+channel, note, velocity
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W1
;		W4
;		ChannelNumber
;		NoteNumber
;		VelocityNumber
;		TMR2
;	Calls
;		SendMidiCharacter
;		TriggerPad_NoteOFF
;		OutputMessage8
;
;***************************************;***************************************
;
TriggerPad_NoteOFF:
		clr		W0
		mov.b	MyPad,	WREG			; get the Pad#
		dec.b	W0,		W0		
		rlnc	W0,		W0				; times 2
		rlnc	W0,		W0				; times 2
		mov		#P1,	W4				; P1 is base address
		add.b	W0,		W4,		W4		; add in offset
		mov.b	[W4++],	W1				; channel value at offset
		dec.b	W1,		W1				; minus 1
		mov		W1,		W0		
		mov.b	WREG,	ChannelNumber
;	
		mov.b	#0x80,	W0				; status for Note ON
		add.b	W1,		W0,		W0		; total status including channel
		call	SendMidiCharacter
;
		mov.b	[W4],	W0				; note value at offset
		mov.b	WREG,	NoteNumber	
		call	SendMidiCharacter
;
		mov.b	#0,		W0				; velocity of 0
		mov.b	WREG,	VelocityNumber
		call	SendMidiCharacter										
;
		call	OutputMessage8
;			
TriggerPad_NoteOFF_Exit:
		clr		TMR2					; reset the Active Sense Timer
		return
;}
;***************************************;***************************************
;
;	Subroutine: Change Channel Setting  (offset+0)
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W4
;		MyValue
;		Channel entry @ P(MyPad) offset 0
;	Calls
;		none
;
;***************************************;***************************************
;{
ChangeChannelSetting:
		clr		W0
		mov.b	MyPad,	WREG
		dec.b	W0,		W0
		rlnc	W0,		W0
		rlnc	W0,		W0
		mov		#P1,	W4
		add.b	W0,		W4,		W4
		mov.b	MyValue,	WREG
		mov.b	W0,		[W4]	
		return
;}
;***************************************;***************************************
;
;	Subroutine: Change Note Setting  (offset+1)
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W4
;		MyValue
;		Note entry @ P(MyPad) offset 1
;	Calls
;		none
;
;***************************************;***************************************
;{
ChangeNoteSetting:
		clr		W0
		mov.b	MyPad,	WREG
		dec.b	W0,		W0
		rlnc	W0,		W0
		rlnc	W0,		W0
		mov		#P1,	W4
		add.b	W0,		W4,		W4
		mov.b	MyValue,	WREG
		inc		W4,		W4				; Note offset
		mov.b	W0,		[W4]	
		return
;}
;***************************************;***************************************
;
;	Subroutine: Change Instrument Setting  (offset+2)
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W4
;		MyValue
;		Instrument entry @ P(MyPad) offset 0
;	Calls
;		none
;
;***************************************;***************************************
;{
ChangeInstrumentSetting:
		clr		W0
		mov.b	MyPad,	WREG
		dec.b	W0,		W0
		rlnc	W0,		W0
		rlnc	W0,		W0
		mov		#P1,	W4
		add.b	W0,		W4,		W4
		mov.b	MyValue,	WREG
		inc2	W4,		W4				; Instrument offset
		mov.b	W0,		[W4]	
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Change Velocity Setting (offset+3)
;
;	Required
;		MyPad	=	Pad#
;	Changes
;		W0
;		W4
;		MyValue
;		Velocity entry @ P(MyPad) offset 0
;	Calls
;		none
;
;***************************************;***************************************
;{
ChangeVelocitySetting:
		clr		W0
		mov.b	MyPad,	WREG
		dec.b	W0,		W0
		rlnc	W0,		W0
		rlnc	W0,		W0
		mov		#P1,	W4
		add.b	W0,		W4,		W4
		mov.b	MyValue,	WREG
		inc		W4,		W4
		inc2	W4,		W4				; Velocity offset
		mov.b	W0,		[W4]	
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Output Pad Settings	-	(Terminal Display)
;	Format:	"Pad:v, Channel:w, Note:x, Vel:y, Instr:z" 
;
;	Required
;		zero terminated string (Pad)
;		zero terminated string (Channel)
;		zero terminated string (Note)
;		zero terminated string (Instrument)
;		zero terminated string (Velocity)
;	Changes
;		W0
;		W1
;		W4
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{        
OutputPadSettings:
		mov		#tblpage(Pad), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Pad), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	MyPad,	WREG
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov.b	#' ',	W0
		call	SendCharacter
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table 
		call	SendMessage
;
		clr		W0
		mov.b	MyPad,	WREG
		dec.b	W0,		W0
		rlnc	W0,		W0
		rlnc	W0,		W0
		mov		#P1,	W4
		add.b	W0,		W4,		W4
		mov.b	[W4++],	W0	
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov.b	#' ',	W0
		call	SendCharacter
;
		mov		#tblpage(Note), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Note), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	[W4++],	W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov.b	#' ',	W0
		call	SendCharacter
;
		mov		#tblpage(Instrument), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Instrument), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	[W4++],	W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov.b	#' ',	W0
		call	SendCharacter
;
		mov		#tblpage(Velocity), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Velocity), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	[W4],	W0		
		call	SendValue		
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output Entry Error Message	-	(Terminal Display)
;	Format:	"Pad:w, Channel:x, Note: y, Voice: z" 
;
;	Required
;		zero terminated string (EntryError)
;	Changes
;		W1
;	Calls
;		SendMessage
;		SendCRLF
;
;***************************************;***************************************
;{       
OutputEntryError:
		mov		#tblpage(EntryError), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(EntryError), W1	; Make a pointer to table 
		call	SendMessage
;
		call	SendCRLF
;
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Output Message8	-	(Terminal Display)
;	Format:	"Note Off (Channel:x, Note:y, Velocity:z)" 
;
;	Required
;		zero terminated string (Message8)
;		zero terminated string (Channel)
;		zero terminated string (Note)
;		zero terminated string (Velocity)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessage8:
		mov		#tblpage(Message8), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Message8), W1	; Make a pointer to table 
		call	SendMessage
;		
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table 
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Note), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Note), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	NoteNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Velocity), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Velocity), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	VelocityNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return 
;}
;***************************************;***************************************
;
;	Subroutine: Output Message9	-	(Terminal Display)
;	Format:	"Note On (Channel:x, Note:y, Velocity:z)" 
;
;	Required
;		zero terminated string (Message9)
;		zero terminated string (Channel)
;		zero terminated string (Note)
;		zero terminated string (Velocity)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessage9:
		mov		#tblpage(Message9), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Message9), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Note), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Note), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	NoteNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Velocity), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Velocity), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	VelocityNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageA	-	(Terminal Display)
;	Format:	"Channel: x, Note: y, Pressure: z"
;
;	Required
;		zero terminated string (MessageA)
;		zero terminated string (Channel)
;		zero terminated string (Note)
;		zero terminated string (Pressure)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
; 
;***************************************;***************************************
;{
OutputMessageA:
		mov		#tblpage(MessageA), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageA), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Note), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Note), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	NoteNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Pressure), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Pressure), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	PressureNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageB	-	(Terminal Display)
;	Format:	"Control Change (Controller:x, Value:y)" 
;
;	Required
;		zero terminated string (MessageB)
;		zero terminated string (Controller)
;		zero terminated string (Value)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageB:
		mov		#tblpage(MessageB), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageB), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Controller), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Controller), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ControllerNumber,	WREG
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Value), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Value), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ValueNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageC	-	(Terminal Display)
;	Format:	"Program Change (Channel:x, Program:y)" 
;
;	Required
;		zero terminated string (MessageC)
;		zero terminated string (Channel)
;		zero terminated string (Program)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageC:
		mov		#tblpage(MessageC), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageC), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Program), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Program), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ProgramNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageD	-	(Terminal Display)
;	Format:	"Pressure Change (Channel:x, Pressure:y)" 
;
;	Required
;		zero terminated string (MessageD)
;		zero terminated string (Channel)
;		zero terminated string (Pressure)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageD:
		mov		#tblpage(MessageD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageD), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Pressure), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Pressure), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	PressureNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageE	-	(Terminal Display)
;	Format:	"Pitch Wheel (Channel:x, Transpose:y)"
;
;	Required
;		zero terminated string (MessageE)
;		zero terminated string (Controller)
;		zero terminated string (Transpose)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCharacter
;		SendCRLF
; 
;***************************************;***************************************
;{
OutputMessageE:
		mov		#tblpage(MessageE), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageE), W1	; Make a pointer to table
		call	SendMessage
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ChannelNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		mov.b	#',',	W0
		call	SendCharacter
;
		mov		#tblpage(Transpose), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Transpose), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TransposeNumber,	WREG
		call	SendValue
;
		mov.b	#')',	W0
		call	SendCharacter
;
		call	SendCRLF
;				
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageF1	-	(Terminal Display)
;
;	Required
;		zero terminated string (MessageF1)
;		zero terminated string (TimeCode)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageF1:
		mov		#tblpage(MessageF1), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF1), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		mov		#tblpage(TimeCode), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(TimeCode), W1	; Make a pointer to table
		call	SendMessage
;
		mov		TimeCodeNumber,	WREG
		call	SendValue
;
		call	SendCRLF
;		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageF2	-	(Terminal Display)
;
;	Required
;		zero terminated string (MessageF2)
;		zero terminated string (MidiBeat)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageF2:
		mov		#tblpage(MessageF2), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF2), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		mov		#tblpage(MidiBeat), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MidiBeat), W1	; Make a pointer to table
		call	SendMessage
;
		mov		MidiBeatNumber,	WREG
		call	SendValue
;
		call	SendCRLF
;		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Output MessageF3	-	(Terminal Display)
;
;	Required
;		zero terminated string (MessageF3)
;		zero terminated string (Song)
;	Changes
;		W0
;		W1
;	Calls
;		SendMessage
;		SendValue
;		SendCRLF
;
;***************************************;***************************************
;{
OutputMessageF3:
		mov		#tblpage(MessageF3), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MessageF3), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		mov		#tblpage(Song), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Song), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	SongNumber,	WREG
		inc.b	W0,		W0		
		call	SendValue
;
		call	SendCRLF
;		
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Send a Message	-	ROM string to U2TXBuffer (Terminal Display)
;
;	Requires
;		W1	=	Message Pointer
;	Changes
;		U2TXCharacterIn
;		W0
;		W1
;	Calls
;		U2TXBuffer_Add (Destroys none)
;
;***************************************;***************************************
;{
SendMessage:
;
SendMessage_Loop: 
		tblrdl.b	[W1++],	W0			; Load the first data value
		cp0.b	W0
		bra		Z,	SendMessage_Exit
;
		mov.b	WREG,	U2TXCharacter
		call	U2TXBuffer_Add
;
		goto	SendMessage_Loop	
;		
SendMessage_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send a Value (ddd) to U2TXBuffer	-	(Terminal Display)
;
;	Requires
;		W0	=	Value
;	Changes
;		W0
;		W1
;	Calls
;		SendCharacter 
;
;***************************************;***************************************
;{
SendValue:
		push.d	W2
		clr		W2
		mov.b	W0,	W2
		mov		#100,	W3
		repeat 	#17		
		div.uw	W2,	W3
		add		#0x30,	W0
		call	SendCharacter
;
		clr		W2
		mov.b	W1,	W2
		mov		#10,	W3
		repeat 	#17		
		div.uw	W2,	W3
		add		#0x30,	W0
		call	SendCharacter
;
		mov.b	W1,		W0
		add		#0x30,	W0
		call	SendCharacter		
		
		pop.d	W2
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send a Character to U2TXBuffer	-	(Terminal Display)
;
;	Requires
;		W0	=	Character
;	Changes
;		U2TXCharacter
;	Calls
;		U2TXBuffer_Add (Destroys none)
;
;***************************************;***************************************
;{
SendCharacter:
		mov.b	WREG,	U2TXCharacter
		call	U2TXBuffer_Add
;		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send a CRLF to U2TXBuffer	-	(Terminal Display)
;
;	Requires
;		none
;	Changes
;		W0
;	Calls
;		SendCharacter 
;
;***************************************;***************************************
;{
SendCRLF:		
		mov.b	#0x0D,	W0
		call	SendCharacter
;						
		mov.b	#0x0A,	W0
		call	SendCharacter
;
		return	
;}
;***************************************;***************************************
;
;	Subroutine: Send a character 	-	(Midi)
;
;	Requires
;		W0	=	character to send
;	Changes
;		U1TXCharacter
;	Calls
;		U1TXBuffer_Add 
;
;***************************************;***************************************
;{
SendMidiCharacter:
		mov.b	WREG,	U1TXCharacter
		call	U1TXBuffer_Add
;		
SendMidiCharacter_Exit:
		return
;}
;***************************************;**************************************
;
;	Subroutine:	InitializeLCD	-	Set up LCD (4line by 20 character)
;
;	Requires:
;		zero terminated string (SignOn1)
;		zero terminated string (SignOn2)
;		zero terminated string (SignOn3)
;		zero terminated string (SignOn4)
;		zero terminated string (SignOn5)
;		Flags,	#RB5State
;	Changes:
;		W0
;		PR1
;		W1
;		Flags,	#LCDFound
;		LCDData
;		LCDRow
; 	Calls:
;		Delay
;		LCD_W_8BitControl
;		LCD_W_Control
;		PositionCursor
;		WriteROMStringToLCD
;
;***************************************;**************************************
;{
InitializeLCD:
		mov		#1563,	W0				; number of 16uS delays (25mS)
		mov		W0,		PR1
		call	Delay   				; go wait
;	
		mov		#0x30,	W0				; Set up 8-bit Mode
		mov.b	WREG,	LCDData
		call	LCD_W_8BitControl		; Write to the LCD's control register
;		
		mov 	#625,	W0				; number of 16uS delays (10mS)
		mov		W0,		PR1
		call	Delay   				; go wait
;		
		mov		#0x30,	W0				; Set up 8-bit Mode
		mov.b	WREG,	LCDData
		call	LCD_W_8BitControl		; Write to the LCD's control register
;	
		mov 	#13,	W0				; number of 16uS delays (200uS)
		mov		W0,		PR1
		call	Delay   				; go wait
;		
		mov		#0x30,	W0				; Set up 8-bit Mode
		mov.b	WREG,	LCDData
		call	LCD_W_8BitControl		; Write to the LCD's control register
;	
		mov 	#13,	W0				; number of 16uS delays (200uS)
		mov		W0,		PR1
		call	Delay   				; go wait
;		
		mov		#0x20,	W0				; Set up 4-bit Mode
		mov.b	WREG,	LCDData
		call	LCD_W_8BitControl			; Write to the LCD's control register				
;
		mov 	#13,	W0				; number of 100uS delays (200uS)
		mov		W0,		PR1
		call	Delay   				; go wait
;
InitializeLCD_LookForLCD:
		call	LCD_R_Control			; go read the LCD control port (busy)
		mov		TEMP,	W0
		mov		#0xFF,	W1
		cp		W0,		W1
		bra		Z,		Init_LCD_Exit
;
		bset	Flags,	#LCDFound
		mov		#0x28,	W0				; 4-bit Mode, 2-Line, 5x8 character Mode
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register			
;
		mov		#0x10,	W0				; no display shift
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register
;
		mov  	#0x0C,	W0				; Display ON, Cursor OFF, Blink OFF
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register
;
		mov  	#0x06,	W0				; Shift Cursor Right
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register	
;
		mov  	#0x01,	W0				; Clear Display
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register
;
		mov		#1,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		btsc	Flags,	#RB5State		; skip next if JP2 is ON, I'm a monitor
		goto	Init_LCD_DrumKit
;
Init_LCD_Monitor:			
		mov		#tblpage(SignOn1),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn1),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
		goto	Init_LCD_Continue
;				
Init_LCD_DrumKit:
		mov		#tblpage(SignOn6),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn6),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
Init_LCD_Continue:
		mov		#2,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		mov		#tblpage(SignOn2),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn2),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
		mov		#3,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		mov		#tblpage(SignOn3),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn3),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
;
		mov		#4,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;
		btss	Flags,	#RB5State		; skip next if RB5State=1 (Drum Kit)
		goto	Init_LCD_MonitorAgain		; else Monitor
;
Init_LCD_DrumKitAgain:		
		mov		#tblpage(SignOn5),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn5),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
		goto	Init_LCD_Wait
;
Init_LCD_MonitorAgain:		
		mov		#tblpage(SignOn4),	W0	; StringTable Pointer		
		mov		W0,		TBLPAG
		mov		#tbloffset(SignOn4),	W1			
		call	WriteROMStringToLCD		; write ROM message to LCD
		goto	Init_LCD_Wait				
;
Init_LCD_Wait:
		mov 	#62500,	W0				; number of 16uS delays (1S)
		mov		W0,		PR1
		call	Delay   				; go wait
;
Init_LCD_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine:	Position Cursor (row)	(LCD)
;
;	Requires:
;		LCDRow
;		LCDData
;	Changes:
;		W0
;		W1
;		LCDData
; 	Calls:
;		LCD_W_Control
;
;***************************************;***************************************
;{
PositionCursor:
		mov.b	LCDRow,	WREG
		mov.b	#1,		W1
		cp.b	W0,		W1				; Is it 1?
		bra		nz,		Row2			; branch if not
;
Row1:
		bset	LCDData,	#7			; set DD RAM Address
		call	LCD_W_Control			; go write cursor position to LCD 
		bra		PositionCursor_Exit
;
Row2:
		mov.b	#2,		W1
		cp.b	W0,		W1				; Is it 1?
		bra		nz,		Row3			; branch if not
;		
		bset	LCDData,	#6			; second row
		bset	LCDData,	#7			; set DD RAM Address
		call	LCD_W_Control			; go write cursor position to LCD 
		bra		PositionCursor_Exit	
;
Row3:
		mov.b	#3,		W1
		cp.b	W0,		W1				; Is it 1?
		bra		nz,		Row4			; branch if not
		
		bset	LCDData,	#7			; set DD RAM Address
		mov		#DisplayWidth,	W0
		add.b	LCDData
		call	LCD_W_Control			; go write cursor position to LCD 
		bra		PositionCursor_Exit
;
Row4:
		bset	LCDData,	#6			; second row
		bset	LCDData,	#7			; set DD RAM Address
		mov		#DisplayWidth,	W0
		add.b	LCDData		
		call	LCD_W_Control			; go write cursor position to LCD 
		bra		PositionCursor_Exit			
;
PositionCursor_Exit:		
		return		
;}
;***************************************;***************************************
;
;	Subroutine:	Delay 96uS	(Timer 1)
;
;	Requires:
;		none
;	Changes:
;		IFS0,	#T1IF
;		TMR1
; 	Calls:
;		none
;
;***************************************;***************************************
;{
Delay:
		bclr	IFS0,	#T1IF			; clear any overflows	
		clr		TMR1
;
Delay_Loop:
		btss	IFS0,	#T1IF
		goto	Delay_Loop
;
		return				
;}
;***************************************;**************************************
;
;	Subroutine:	LCD_W_8BitControl	-	write a nibble
;
;	Requires:
;		LCDData
;	Changes:
;		W0
;		W2
;		W3
;		TRISB
;		LATB,	#RS
;		LATA,	#R_WR
; 	Calls:
;		OutputANibble
;
;***************************************;**************************************
;{
LCD_W_8BitControl:
		mov		#0xF0FF,W2				; get 0xF0FF into W2
		mov		TRISB,	W3				; get TRISB into W3
		and		W2,		W3,		W2		; make RB11:8 outputs in W2 
		mov		W2,		TRISB			; replace TRISB
		bclr	LATB,	#RS				; RS=Control
		nop
		nop
		bclr	LATA,	#R_WR			; RD_W=Write
;	
		mov.b	LCDData,WREG			; get high nibble
		swap.b	W0						; swap upper and lower nibbles
		swap	W0						; swap lower and lower bytes		
		call	OutputANibble			; go write upper nibble of data to LCD
;
		return	
;}
;***************************************;**************************************
;
;	Subroutine:	OutputANibble	-	Write a nibble to the control register
;
;	Requires:
;		W0	=	Nibble to send
;	Changes:
;		W2
;		W3
;		TRISB
;		LATB,	#E
;		LATB
; 	Calls:
;		none
;
;***************************************;**************************************
;{
OutputANibble:
		mov		TRISB,	W2				; get TRISB into W2
		mov		#0xF0FF,W3				; get 0xF0F into W3
		and		W2,	W3,	W2				; clear B11:8 of W2 (outputs)
		mov		W2,	TRISB				; replace TRISB			
;	
		mov		PORTB,	W2				; get PORTB into W2
		mov		#0xF0FF,W3				; get 0xFFF0 into W3				
		and		W2,	W3,	W2				; clear lower bits of W2 	
		ior		W0,		W2,   	W2		; add lower bit data from W0 into W2
		mov		W2,	LATB				; replace data on LATB
		nop
		nop
		nop
		nop
 		bset	LATB,	#E				; E=Enable Strobe
		nop
		nop
		nop
		nop
		bclr	LATB,	#E				; E=Disable Strobe
		nop
		nop
		nop
		nop
		return
;}
;***************************************;***************************************
;
;	Subroutine:	WriteROMStringToLCD	-	20 character string is written to LCD
;
;	Requires:
;		W1	=	Pointer to ROM String
;	Changes:
;		W0
;		W1
;		LCDData
; 	Calls:
;		LCD_W_Data
;		
;***************************************;***************************************
;{
WriteROMStringToLCD:
		tblrdl.b	[W1++],	W0			; get ROM table word into W1
		cp0.b	W0					; get <null> into W2
		bra		Z,	WriteROMStringToLCD_Exit	; branch if lower byte=null
;
WriteROMStringToLCD_Loop:	
		mov.b	WREG,	LCDData			; get the ROM character into LCDData
		call	LCD_W_Data				; write LCDData to LCD dat register
;
		tblrdl.b	[W1++],	W0			; get ROM table charaters into W1
		cp0.b	W0						; get <null> into W2
		bra		Z,	WriteROMStringToLCD_Exit	; branch if lower byte=0x0D
;
		goto	WriteROMStringToLCD_Loop
;
WriteROMStringToLCD_Exit:	
		return
;}
;***************************************;**************************************
;
;	Subroutine:	LCD_W_Data	-	write byte to LCD
;
;	Requires:
;		LCDData
;	Changes:
;		W0
;		W2
;		W3
;		TRISB
;		LATB,	#RS
;		LATA, 	#R_WR
; 	Calls:
;		Chk4Busy
;		OutputANibble
;			
;***************************************;**************************************
;{	
LCD_W_Data:
		call	Chk4Busy				; wait for LCD to be ready (Busy bit=0)
;
		mov		#0xF0FF,W2				; get 0xF0FF into W2
		mov		TRISB,	W3				; get TRISB into W3
		and		W2,		W3,		W2		; lower nibble of W2 as output 
		mov		W2,		TRISB			; replace TRISB
		bset	LATB,	#RS				; RS=Data
		nop
		nop
		bclr	LATA, 	#R_WR			; RD_W=Write
;	
		mov.b	LCDData,WREG			; LCDData into W0
		swap.b	W0						; swap upper and lower nibbles of W0
		mov		#0x000F,W2				; get 0c000F into W2
		and.b	W0,		W2,		W0		; just lower nibble of W0
		swap	W0						; move lower to upper byte		
		call	OutputANibble			; Write a nibble to the LCD
;
		mov.b	LCDData,WREG			; LCDData into W0
		mov		#0x000F,W2				; get 0c000F into W2
		and.b	W0,		W2,		W0		; just lower nibble of W0
		swap	W0						; move lower to upper byte		
		call	OutputANibble			; Write a nibble to the LCD
		return
;}
;***************************************;**************************************
;
;	Subroutine:	LCD_W_Control	-	write to the LCD's control register
;
;	Requires:
;		LCDData
;	Changes:
;		W0
;		W2
;		W3
;		TRISB
;		LATB,	#RS
;		LATA, 	#R_WR
; 	Calls:
;		Chk4Busy
;		OutputANibble
;
;***************************************;**************************************
;{
LCD_W_Control:
		call	Chk4Busy				; go wait for the LCD to be ready
;		
		mov		#0xF0FF,W2				; get 0xF0FF into W0
		mov		TRISB,	W3				; get TRISB into W3
		and		W2,		W3,		W2		; make the lower nibble in W21 outputs 
		mov		W2,		TRISB			; replace TRISB
		bclr	LATB,	#RS				; RS=Control
		nop
		nop
		bclr	LATA,	#R_WR			; RD_W=Write
;
		clr		W0	
		mov.b	LCDData,WREG			; get LCDData into W0
		swap.b	W0						; swap upper and lower nibbles of W0
		mov		#0x000F,W2				; get 0x000F into W2
		and.b	W0,		W2,		W0		; just lower nibble in W2 (was upper)
		swap	W0						; swap upper and lower bytes of W0		
		call	OutputANibble			; write W1 to LCD data register
;
		clr		W0
		mov.b	LCDData,WREG			; get LCDData into W0
		mov		#0x000F,W2				; get 0x000F into W2
		and.b	W0,		W2,		W0		; just lower nibble in W0
		swap	W0						; swap upper and lower bytes of W0		
		call	OutputANibble			; write W1 to LCD data register
		return	
;}
;***************************************;***************************************
;	
;	Subroutine:	Chk4Busy	-	read from the LCD's control register 
;
;	Requires:
;		none
;	Changes:
;		none
; 	Calls:
;		LCD_R_Control
;
;***************************************;***************************************
;{	
Chk4Busy:
		call	LCD_R_Control			; go read the LCD control port (busy)
		btsc	TEMP,	#Busy
		goto	Chk4Busy				; keep checking
;
		return
;}
;***************************************;**************************************
;
;	Subroutine:	LCD_R_Control	-	read from the LCD's control register
;
;	Requires:
;		none
;	Changes:
;		W2
;		W3
;		TRISB
;		LATB,	#RS
;		LATA,	#R_WR
;		LATB,	#E
;		TEMP
; 	Calls:
;		none
;
;***************************************;**************************************
;{	
LCD_R_Control:
		mov		#0x0F00,W2				; get 0x0F00 into W2
		mov		TRISB,	W3				; get TRISB into W3
		ior		W2,	W3,	W3				; make lower nibble of W3 inputs
		mov		W3,	TRISB				; replace TRISB
		bclr	LATB,	#RS				; RS=Control
		nop
		nop
		bset	LATA,	#R_WR			; R_WR=Read
		nop
		nop
		nop
		nop
		bset	LATB,	#E				; E=Enable Strobe
		nop
		nop
		nop
		nop
		mov		PORTB,	W2				; get PORTB data
		swap	W2						; move upper to lower byte		
		swap.b	W2						; move lower to upper nibble
		and		#0x00F0,W2				; just upper nibble into W2
		mov		W2,	TEMP				; save W10 temporarily
		bclr	LATB,	#E				; E=Disable Strobe
		nop
		nop
		nop
		nop
		bset	LATB,	#E				; E=Enable Strobe
		nop
		nop
		nop
		nop
		mov		PORTB,	W2				; get PORTB data
		swap	W2						; move upper to lower byte					
		and		#0x000F,W2				; just the lower nibble
		mov		TEMP,	W3				; retrieve temporarily stored
										; upper nibble into W3
		ior		W2,	W3,	W2				; combine W11 into W2
		mov		W2,	TEMP				; and save into TEMP
		bclr	LATB,	#E				; E=Disable Strobe
		nop
		nop
		nop
		nop					
		return
;}
;***************************************;**************************************
;
;	Subroutine:	Update the LCD	-	Display present parameters on LCD
;									Includes positioning blinking cursor
;		 							as Mode indicator
;
;	Requires:
;		Flags,	#LCDFound
;		MyPad
;		MyChannel
;		MyNote
;		MyVelocity
;		MyInstrument
;		MyBPM
;		MyMode
;		zero terminated string (Pad)
;		zero terminated string (Channel)
;		zero terminated string (Note)
;		zero terminated string (Velocity)
;		zero terminated string (Instrument)
;		zero terminated string (MyBPM)
;	Changes:
;		W0
;		W1
;		TRISB
;		LATB,	#RS
;		LATA,	#R_WR
;		LATB,	#E
;		TEMP
;		LCDBufferPointer
;		LCDRow
;		LCDData
; 	Calls:
;		ClearLCDBuffer
;		WriteROMStringToRAM
;		SendValueToRAM
;		PositionCursor
;		WriteRAMStringToLCD
;		LCD_W_Control
;
;***************************************;**************************************
;{
UpdateLCD:
		btss	Flags,	#LCDFound
		goto	UpdateLCD_Exit
;
UpdateLCD_ClearScreen:
;		mov  	#0x01,	W0				; Clear Display
;		mov.b	WREG,	LCDData
;		call	LCD_W_Control			; Write to the LCD's control register
;
UpdateLCD_Pad_Channel:
		call	ClearLCDBuffer
;
		mov		#LCDBuffer+1,	W0
		mov		W0,	LCDBufferPointer		
;		
		mov		#tblpage(Pad), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Pad), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyPad,	WREG
		call	SendValueToRAM
;
		mov		#LCDBuffer+9,	W0
		mov		W0,	LCDBufferPointer
;
		mov		#tblpage(Channel), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Channel), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyChannel,	WREG
		call	SendValueToRAM
;
		mov		#1,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;	
		call	WriteRAMStringToLCD	
;
UpdateLCD_Note_Velocity:
		call	ClearLCDBuffer
;		
		mov		#tblpage(Note), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Note), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyNote,	WREG
		call	SendValueToRAM
;
		mov		#LCDBuffer+13,	W0
		mov		W0,	LCDBufferPointer
;
		mov		#tblpage(Velocity), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Velocity), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyVelocity,	WREG
		call	SendValueToRAM
;
		mov		#2,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;	
		call	WriteRAMStringToLCD
;
UpdateLCD_Instrument_BPM:
		call	ClearLCDBuffer
;
		mov		#tblpage(Instrument), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Instrument), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyInstrument,	WREG
		call	SendValueToRAM
;
		mov		#LCDBuffer+13,	W0
		mov		W0,	LCDBufferPointer		
;		
		mov		#tblpage(Beats), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Beats), W1	; Make a pointer to table 
		call	WriteROMStringToRAM		; write ROM message to LCD
;
		mov.b	MyBeats,	WREG
		call	SendValueToRAM
;
		mov		#LCDBuffer+16,	W0
		mov		W0,	LCDBufferPointer
;
		mov		#3,		W0
		mov.b	WREG,	LCDRow				
		clr.b	LCDData					; LCDData=0 (no offset to position)
		call	PositionCursor			; go do it
;	
		call	WriteRAMStringToLCD		
;
UpdateLCD_Cursor:
;
		mov  	#0x0D,	W0				; Display ON, Cursor OFF, Blink ON
		mov.b	WREG,	LCDData
		call	LCD_W_Control			; Write to the LCD's control register
;
		mov		#0,		W1
		mov.b	MyMode,	WREG
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode0
;
		mov		#1,		W1
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode1		
;
		mov		#2,		W1
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode2
;
		mov		#3,		W1
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode3
;
		mov		#4,		W1
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode4
;
		mov		#5,		W1
		cp.b	W0,		W1
		bra		Z,		UpdateLCD_Cursor_Mode5	
;
UpdateLCD_Cursor_Mode6:
		mov.b	#Row4Column1+7,	W0	; row 4 position 8
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit			
;
UpdateLCD_Cursor_Mode5:
		mov.b	#Row3Column1+13,	W0	; row 3 position 14
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit	
;
UpdateLCD_Cursor_Mode4:
		mov.b	#Row3Column1+0,	W0		; row 3 position 1
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit
;
UpdateLCD_Cursor_Mode3:
		mov.b	#Row2Column1+13,	W0	; row 2 position 14
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit
;
UpdateLCD_Cursor_Mode2:
		mov.b	#Row2Column1+0,	W0		; row 2 position 1
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit
;
UpdateLCD_Cursor_Mode1:
		mov.b	#Row1Column1+9,	W0		; row 1 position 10
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;
		goto	UpdateLCD_Exit
;
UpdateLCD_Cursor_Mode0:
		mov.b	#Row1Column1+1,	W0		; row 1 position 2
		mov.b	WREG,	LCDData
		call	LCD_W_Control
;		
		goto	UpdateLCD_Exit																
;		
UpdateLCD_Exit:				
		return
;}
;***************************************;***************************************
;
;	Subroutine:	ClearLCDBuffer	-	fill LCDbuffer with spaces
;
;	Requires:
;		none
;	Changes:
;		W0
;		W1
;		W2
;		LCDBufferPointer
; 	Calls:
;		none
;			
;***************************************;***************************************
;{
ClearLCDBuffer:	
		push	W2	
		mov		#LCDBuffer,	W1
		mov		#LCDBuffer+20,	W2		
		mov.b	#' ',	W0
;
ClearLCDBuffer_Loop:			
		mov.b	W0,	[W1++]
		cp		W1,		W2
		bra		NZ,		ClearLCDBuffer_Loop			
;
ClearLCDBuffer_Exit:
		mov		#LCDBuffer,	W1
		mov		W1,		LCDBufferPointer
		pop		W2
		return				
;}
;***************************************;***************************************
;
;	Subroutine:	WriteROMStringToRAM	-	move ROM string to LCDbuffer
;
;	Requires:
;		LCDBufferPointer
;		W1	=	Pointer to ROM String
;	Changes:
;		W0
;		W1
;		W2
;		LCDBufferPointer
; 	Calls:
;		none
;			
;***************************************;***************************************
;{
WriteROMStringToRAM:
		push	W2
		mov		LCDBufferPointer,	W2	; get RAM buffer pointer
;
WriteROMStringToRAM_Loop:		
		tblrdl.b	[W1++],	W0			; get ROM table word into W0
		cp0.b	W0						; W0 = <null>?
		bra		Z,	WriteROMStringToRAM_Exit	; branch if W0=<null>
;
		mov.b	W0,	[W2++]			; movee ROM character to RAM buffer
		goto	WriteROMStringToRAM_Loop
;
WriteROMStringToRAM_Exit:
		mov		W2,		LCDBufferPointer	; replace RAM buffer pointer
		pop		W2	
		return
;}		
;***************************************;***************************************
;
;	Subroutine:	WriteRAMStringToLCD	-	20 character string is written to LCD
;
;	Requires:
;		none
;	Changes:
;		W0
;		W1
;		LCDData
; 	Calls:
;		LCD_W_Data
;			
;***************************************;***************************************
;{
WriteRAMStringToLCD:
		mov		#LCDBuffer,	W1
;
WriteRAMStringToLCD_Loop:	
		mov.b	[W1++],	W0
		mov.b	WREG,	LCDData			; get the RAM character into LCDData
		call	LCD_W_Data				; write LCDData to LCD dat register
;
		mov		#LCDBuffer+20,	W0
		cp		W0,		W1
		bra		Z,WriteRAMStringToLCD_Exit	; branch if at end of buffer
;
		goto	WriteRAMStringToLCD_Loop
;
WriteRAMStringToLCD_Exit:	
		return
;}		
;***************************************;***************************************
;
;	Subroutine:	SendValueToRAM	-	move a value to LCD buffer (ddd)
;	Requires:
;		W0	=	Value
;	Changes:
;		W0
;		W1
; 	Calls:
;		SendCharacterToRAM
;			
;***************************************;***************************************
;{
SendValueToRAM:
		push.d	W2
		clr		W2
		mov.b	W0,	W2
		mov		#100,	W3
		repeat 	#17		
		div.uw	W2,	W3
		add		#0x30,	W0
		call	SendCharacterToRAM
;
		clr		W2
		mov.b	W1,	W2
		mov		#10,	W3
		repeat 	#17		
		div.uw	W2,	W3
		add		#0x30,	W0
		call	SendCharacterToRAM
;
		mov.b	W1,		W0
		add		#0x30,	W0
		call	SendCharacterToRAM		
		
		pop.d	W2
		return
;}		
;***************************************;***************************************
;
;	Subroutine:	SendCharacterToRAM	-	character to LCD buffer
;
;	Requires:	
;		W0	=	Character
;		LCDBufferPointer	=	Position for character
;	Changes:
;		W0
;		W1
;		LCDBufferPointer
; 	Calls:
;		none
;			
;***************************************;***************************************
;{
SendCharacterToRAM:
		push	W1
		mov		LCDBufferPointer,	W1	; get RAM buffer pointer
		mov.b	W0,		[W1++]
		mov		W1,		LCDBufferPointer	; replace RAM buffer pointer
		pop		W1
		return								
;}
;***************************************;***************************************
;
;	UART1 TX Interrupt Service Routine
;
;	Requires:	
;		none
;	Changes:
;		U1TXBuffer_Tail
;		IFS0,	#U1TXIF
;		U1TXREG
;		IEC0,	#U1TXIE
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U1TXInterrupt:
        push.D 	W0 						; Save context using double-word PUSH
;        bclr 	IFS0,	#U1TXIF         ; Clear the U1TX Interrupt flag bit.        
;
U1TXInterrupt_Loop:                         	
		mov		U1TXBuffer_Head,	W0
		mov		U1TXBuffer_Tail,	W1
		cp		W0,	W1
		bra		Z,	U1TXInterrupt_NoData	; branch if Head=Tail (no data)      
;
;        btsc 	U1STA,	#UTXBF         	; skip next transmitter has room   
;		goto	U1TXInterrupt_Exit		; else load more into transmit buffer
;
        bclr 	IFS0,	#U1TXIF         ; Clear the U1TX Interrupt flag bit. 
		mov.b	[W1],	W0
		mov.b	WREG,	U1TXREG
		inc		W1,	W1
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1
		bra		NZ,	U1TXInterrupt_NoWrap
;
		mov		#U1TXBuffer,	W1        
;
U1TXInterrupt_NoWrap:
		mov		W1,	U1TXBuffer_Tail
;		goto	U1TXInterrupt_Loop		; else load more into transmit buffer
		goto	U1TXInterrupt_Exit		; leave cause transmit buffer is full	
;
U1TXInterrupt_NoData:
		bclr	IEC0,	#U1TXIE         ; Prevent further U1TX interrupts
		goto  	U1TXInterrupt_Exit
;
U1TXInterrupt_Exit:        
		pop.D 	W0                   	;Retrieve context POP-ping from Stack
        retfie                     		;Return from Interrupt Service routine
;}
;***************************************;***************************************
;
;	UART1 RX Interrupt Service Routine
;
;	Requires:	
;		none
;	Changes:
;		U1RXBuffer_Head
;		IFS0,	#U1RXIF
;		U1TXREG
;		U1STA,	#OERR
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U1RXInterrupt:
        push.D 	W0                  	; Save context using double-word PUSH
        push.D 	W2                  	; Save context using double-word PUSH        
        bclr 	IFS0,	#U1RXIF         ; Clear the U1RX Interrupt flag bit.
;                
		btsc	U1STA,	#OERR			; skip next if no OERR
		goto	U1RXInterrupt_OERR
;		
		btsc	U1STA,	#FERR			; skip next if no FERR
		goto	U1RXInterrupt_FERR		
;		
		btsc	U1STA,	#PERR			; skip next if no PERR
		goto	U1RXInterrupt_PERR	
;
		mov		U1RXBuffer_Head,	WREG
		inc		W0,	W1
		mov		W0,	W2
		mov		#U1RXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1
		bra		NZ,	U1RXInterrupt_NoWrap
;
		mov		#U1RXBuffer,	W1
;
U1RXInterrupt_NoWrap:				
		mov		U1RXBuffer_Tail,	W0
		cp		W0,	W1		
		bra		Z,	U1RXInterrupt_NoRoom
;
		mov.b	U1RXREG,	WREG
		mov.b	W0,	[W2]
		mov		W1,	U1RXBuffer_Head		
;
		goto	U1RXInterrupt_Exit	
;				
U1RXInterrupt_OERR:
		bclr	U1STA,	#OERR
		goto	U1RXInterrupt_Exit
;		
U1RXInterrupt_FERR:
		mov.b	U1RXREG,	WREG		; dummy read to toss out bad data
		goto	U1RXInterrupt_Exit
;		
U1RXInterrupt_PERR:	
		mov.b	U1RXREG,	WREG		; dummy read to toss out bad data	
		goto	U1RXInterrupt_Exit
;
U1RXInterrupt_NoRoom:
		mov.b	U1RXREG,	WREG		; toss out data, no room in U1RXBuffer
		goto	U1RXInterrupt_Exit
;
U1RXInterrupt_Exit:
        pop.D 	W2                   	; Retrieve context POP-ping from Stack
        pop.D 	W0                   	; Retrieve context POP-ping from Stack        
        retfie                     		; Return from Interrupt Service routine
;}
;***************************************;***************************************
;
;	UART2 TX Interrupt Service Routine
;
;	Requires:	
;		none
;	Changes:
;		U2TXBuffer_Tail
;		IFS1,	#U2TXIF
;		U2TXREG
;		IEC1,	#U2TXIE
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U2TXInterrupt:
        push.d 	W0 						; Save context using double-word PUSH
;        bclr 	IFS1,	#U2TXIF         ; Clear the U2TX Interrupt flag bit       
;
U2TXInterrupt_Loop:                         	
		mov		U2TXBuffer_Head,	W0
		mov		U2TXBuffer_Tail,	W1
		cp		W0,	W1
		bra		Z,	U2TXInterrupt_NoData	; branch if Head=Tail (no data)      
;
;        btsc 	U2STA,	#UTXBF         	; skip next transmitter has room   
;		goto	U2TXInterrupt_Exit		; else load more into transmit buffer
;
        bclr 	IFS1,	#U2TXIF         ; Clear the U2TX Interrupt flag bit 		
		mov.b	[W1],	W0
		mov.b	WREG,	U2TXREG 
		inc		W1,	W1
		mov		#U2TXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1
		bra		NZ,	U2TXInterrupt_NoWrap
;
		mov		#U2TXBuffer,	W1        
;
U2TXInterrupt_NoWrap:
		mov		W1,	U2TXBuffer_Tail       
;		goto	U2TXInterrupt_Loop		; else load more into transmit buffer
		goto	U2TXInterrupt_Exit
;
U2TXInterrupt_NoData:
		bclr	IEC1,	#U2TXIE         ; Prevent further U2TX interrupts
		goto  	U2TXInterrupt_Exit
;
U2TXInterrupt_Exit:       
		pop.d 	W0                   	;Retrieve context POP-ping from Stack
        retfie                     		;Return from Interrupt Service routine
;}
;***************************************;***************************************
;
;	UART2 RX Interrupt Service Routine
;
;	Requires:
;		none
;	Changes:
;		U2RXBuffer_Head
;		IFS1,	#U2RXIF
;		U2TXREG
;		U2STA,	#OERR
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U2RXInterrupt:
        push.D 	W0                  	; Save context using double-word PUSH
        push.D 	W2                  	; Save context using double-word PUSH        
        bclr 	IFS1,	#U2RXIF         ; Clear the U2RX Interrupt flag bit.
;                
		btsc	U2STA,	#OERR			; skip next if no OERR
		goto	U2RXInterrupt_OERR
;		
		btsc	U2STA,	#FERR			; skip next if no FERR
		goto	U2RXInterrupt_FERR		
;		
		btsc	U2STA,	#PERR			; skip next if no PERR
		goto	U2RXInterrupt_PERR	
;
		mov		U2RXBuffer_Head,	WREG
		inc		W0,	W1
		mov		W0,	W2
		mov		#U2RXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1		
		bra		NZ,	U2RXInterrupt_NoWrap
;
		mov		#U2RXBuffer,	W1
;
U2RXInterrupt_NoWrap:				
		mov		U2RXBuffer_Tail,	W0
		cp		W0,	W1		
		bra		Z,	U2RXInterrupt_NoRoom
;
		mov.b	U2RXREG,	WREG
		mov.b	W0,	[W2]
		mov		W1,	U2RXBuffer_Head		
;
		goto	U2RXInterrupt_Exit	
;				
U2RXInterrupt_OERR:
		bclr	U2STA,	#OERR
		goto	U2RXInterrupt_Exit
;		
U2RXInterrupt_FERR:
		mov.b	U2RXREG,	WREG		; dummy read to toss out bad data
		goto	U2RXInterrupt_Exit
;		
U2RXInterrupt_PERR:	
		mov.b	U2RXREG,	WREG		; dummy read to toss out bad data	
		goto	U2RXInterrupt_Exit
;
U2RXInterrupt_NoRoom:
		mov.b	U2RXREG,	WREG		; toss out data, no room in U2RXBuffer
		goto	U2RXInterrupt_Exit
;
U2RXInterrupt_Exit:
        pop.D 	W2                   	; Retrieve context POP-ping from Stack 
        pop.D 	W0                   	; Retrieve context POP-ping from Stack        
        retfie                     		; Return from Interrupt Service routine 
;}
;***************************************;***************************************
;
;	Timer2 Interrupt Service Routine	-	Active Sense Message Timing
;
;	tick=4MHz/256=64uS
; 	tick*0x124F=299mS (overflow)
;
;	Requires:
;		none
;	Changes:
;		IFS0,	#T2IF
;		Flags,	#ActiveSenseMessage
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__T2Interrupt:
        push.d 	W0                  	; Save context using double-word PUSH        
        bclr 	IFS0,	#T2IF         	; Clear the U2RX Interrupt flag bit.
        bset	Flags,	#ActiveSenseMessage	; Set Flag to send message
;
T2Interrupt_Exit:
		pop.d	W0        		  
		retfie 
;}
;***************************************;***************************************
;
;	Timer4 Interrupt Service Routine	-	Midi Clock Message Timing
;
;	Requires:
;		none
;	Changes:
;		IFS1,	#T4IF
;		Flags,	Flags,	#MidiClockMessage
;		TMR2
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__T4Interrupt:
        push.d 	W0                  	; Save context using double-word PUSH        
        bclr 	IFS1,	#T4IF         	; Clear the TMR4 Interrupt flag bit.
		btsc	Flags,	#RB6State		; skip next if Midi Clock disabled
        bset	Flags,	#MidiClockMessage	; Set Flag to send message        
;
T4Interrupt_Exit:
		btsc	Flags,	#RB6State		; skip next if Midi Clock disabled
		clr		TMR2					; clear Active Sense Message
		pop.d	W0        		  
		retfie 	
;}
;***************************************;***************************************
;
;	Timer5 Interrupt Service Routine	-	input sampling timer
;
;	Requires:
;		none
;	Changes:
;		IFS1,	#T5IF
;		SR,		#C
;		DirFlags,	#0
;		In1Max
;		TriggerFlags,	#0
;		DirFlags,	#1
;		In2Max
;		TriggerFlags,	#1
;		DirFlags,	#2
;		In3Max
;		TriggerFlags,	#2
;		DirFlags,	#3
;		In4Max
;		TriggerFlags,	#3
;		DirFlags,	#4
;		In5Max
;		TriggerFlags,	#4
;		DirFlags,	#5
;		In6Max
;		TriggerFlags,	#5
;		DirFlags,	#6
;		In7Max
;		TriggerFlags,	#6
;		DirFlags,	#7
;		In8Max
;		TriggerFlags,	#7
;		AD1CON1,	#ASAM
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__T5Interrupt:
        push.d 	W0                  	; Save context using double-word PUSH 
        push.d 	W2                  	; Save context using double-word PUSH                
        bclr 	IFS1,	#T5IF         	; Clear the TMR5 Interrupt flag bit.
;        bset	LATA,	#LED		 	; this is for testing only (scope)
;***************************************
T5Interrupt_ChkIn1Max:
		mov.b	In1Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF0,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn1Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn1Max1:		
		btss	DirFlags,	#0			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn1Max_Disabled	
;
T5Interrupt_ChkIn1Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn1Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn1Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn1Max_Enabled_LT:
		goto	T5Interrupt_ChkIn1Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn1Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn1Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn1Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#0			; Disable Peak Detection
		bset	TriggerFlags,	#0		; Trigger 1
		mov.b	WREG,		In1Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In1Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn1Max_Exit	; all done here
;
T5Interrupt_ChkIn1Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn1Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn1Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn1Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn1Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn1Max_Disabled_LT:
		bset	DirFlags,	#0			; Enable Peak Detection
		goto	T5Interrupt_ChkIn1Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn1Max_Disabled_GT:
		goto	T5Interrupt_ChkIn1Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn1Max_Continue:
		mov.b	WREG,	In1Max		; save the new sample		
;		
T5Interrupt_ChkIn1Max_Exit:	
;***************************************
T5Interrupt_ChkIn2Max:
		mov.b	In2Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF1,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn2Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn2Max1:		
		btss	DirFlags,	#1			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn2Max_Disabled	
;
T5Interrupt_ChkIn2Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn2Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn2Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn2Max_Enabled_LT:
		goto	T5Interrupt_ChkIn2Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn2Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn2Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn2Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#1			; Disable Peak Detection
		bset	TriggerFlags,	#1		; Trigger 2
		mov.b	WREG,		In2Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In2Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn2Max_Exit	; all done here
;
T5Interrupt_ChkIn2Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn2Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn2Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn2Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn2Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn2Max_Disabled_LT:
		bset	DirFlags,	#1			; Enable Peak Detection
		goto	T5Interrupt_ChkIn2Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn2Max_Disabled_GT:
		goto	T5Interrupt_ChkIn2Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn2Max_Continue:
		mov.b	WREG,	In2Max		; save the new sample		
;		
T5Interrupt_ChkIn2Max_Exit:		
;***************************************
T5Interrupt_ChkIn3Max:
		mov.b	In3Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF2,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn3Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn3Max1:		
		btss	DirFlags,	#2			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn3Max_Disabled	
;
T5Interrupt_ChkIn3Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn3Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn3Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn3Max_Enabled_LT:
		goto	T5Interrupt_ChkIn3Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn3Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn3Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn3Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#2			; Disable Peak Detection
		bset	TriggerFlags,	#2		; Trigger 3
		mov.b	WREG,		In3Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In3Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn3Max_Exit	; all done here
;
T5Interrupt_ChkIn3Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn3Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn3Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn3Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn3Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn3Max_Disabled_LT:
		bset	DirFlags,	#2			; Enable Peak Detection
		goto	T5Interrupt_ChkIn3Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn3Max_Disabled_GT:
		goto	T5Interrupt_ChkIn3Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn3Max_Continue:
		mov.b	WREG,	In3Max		; save the new sample		
;		
T5Interrupt_ChkIn3Max_Exit:	
;***************************************   
T5Interrupt_ChkIn4Max:
		mov.b	In4Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF3,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn4Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn4Max1:		
		btss	DirFlags,	#3			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn4Max_Disabled	
;
T5Interrupt_ChkIn4Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn4Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn4Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn4Max_Enabled_LT:
		goto	T5Interrupt_ChkIn4Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn4Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn4Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn4Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#3			; Disable Peak Detection
		bset	TriggerFlags,	#3		; Trigger 4
		mov.b	WREG,		In4Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In4Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn4Max_Exit	; all done here
;
T5Interrupt_ChkIn4Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn4Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn4Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn4Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn4Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn4Max_Disabled_LT:
		bset	DirFlags,	#3			; Enable Peak Detection
		goto	T5Interrupt_ChkIn4Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn4Max_Disabled_GT:
		goto	T5Interrupt_ChkIn4Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn4Max_Continue:
		mov.b	WREG,	In4Max		; save the new sample		
;		
T5Interrupt_ChkIn4Max_Exit:	
;*************************************** 
T5Interrupt_ChkIn5Max:
		mov.b	In5Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF4,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn5Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn5Max1:		
		btss	DirFlags,	#4			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn5Max_Disabled	
;
T5Interrupt_ChkIn5Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn5Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn5Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn5Max_Enabled_LT:
		goto	T5Interrupt_ChkIn5Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn5Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn5Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn5Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#4			; Disable Peak Detection
		bset	TriggerFlags,	#4		; Trigger 5
		mov.b	WREG,		In5Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In5Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn5Max_Exit	; all done here
;
T5Interrupt_ChkIn5Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn5Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn5Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn5Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn5Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn5Max_Disabled_LT:
		bset	DirFlags,	#4			; Enable Peak Detection
		goto	T5Interrupt_ChkIn5Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn5Max_Disabled_GT:
		goto	T5Interrupt_ChkIn5Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn5Max_Continue:
		mov.b	WREG,	In5Max		; save the new sample		
;		
T5Interrupt_ChkIn5Max_Exit:
;***************************************
T5Interrupt_ChkIn6Max:
		mov.b	In6Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF5,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn6Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn6Max1:		
		btss	DirFlags,	#5			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn6Max_Disabled	
;
T5Interrupt_ChkIn6Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn6Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn6Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn6Max_Enabled_LT:
		goto	T5Interrupt_ChkIn6Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn6Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn6Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn6Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#5			; Disable Peak Detection
		bset	TriggerFlags,	#5		; Trigger 6
		mov.b	WREG,		In6Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In6Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn6Max_Exit	; all done here
;
T5Interrupt_ChkIn6Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn6Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn6Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn6Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn6Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn6Max_Disabled_LT:
		bset	DirFlags,	#5			; Enable Peak Detection
		goto	T5Interrupt_ChkIn6Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn6Max_Disabled_GT:
		goto	T5Interrupt_ChkIn6Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn6Max_Continue:
		mov.b	WREG,	In6Max		; save the new sample		
;		
T5Interrupt_ChkIn6Max_Exit:
;***************************************
T5Interrupt_ChkIn7Max:
		mov.b	In7Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF6,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn7Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn7Max1:		
		btss	DirFlags,	#6			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn7Max_Disabled	
;
T5Interrupt_ChkIn7Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn7Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn7Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn7Max_Enabled_LT:
		goto	T5Interrupt_ChkIn7Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn7Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn7Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn7Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#6			; Disable Peak Detection
		bset	TriggerFlags,	#6		; Trigger 7
		mov.b	WREG,		In7Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In7Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn7Max_Exit	; all done here
;
T5Interrupt_ChkIn7Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn7Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn7Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn7Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn7Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn7Max_Disabled_LT:
		bset	DirFlags,	#6			; Enable Peak Detection
		goto	T5Interrupt_ChkIn7Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn7Max_Disabled_GT:
		goto	T5Interrupt_ChkIn7Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn7Max_Continue:
		mov.b	WREG,	In7Max		; save the new sample		
;		
T5Interrupt_ChkIn7Max_Exit:	
;***************************************
T5Interrupt_ChkIn8Max:
		mov.b	In8Max,		WREG
		mov.b	W0,		W1
		mov.b	#Min_OFF,	W3
		mov		ADC1BUF7,	WREG
		swap	W0
		mov		#127,	W4
		cp.b	W0,		W4
		bra		LEU,	T5Interrupt_ChkIn8Max1
;		
		mov		W4,		W0				; force W0 to a maximum of 127
										; (you might want to turn down the
										; volume control on this channel)
;
T5Interrupt_ChkIn8Max1:		
		btss	DirFlags,	#7			; skip next if Peak Detection is Enabled	
		goto	T5Interrupt_ChkIn8Max_Disabled	
;
T5Interrupt_ChkIn8Max_Enabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn8Max_Exit
;		
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn8Max_Enabled_GT	; branch if New Sample>Min_OFF			
;
T5Interrupt_ChkIn8Max_Enabled_LT:
		goto	T5Interrupt_ChkIn8Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn8Max_Enabled_GT:
		cp.b	W0,		W1
		bra		GTU,	T5Interrupt_ChkIn8Max_Enabled_GT_NotYet	; branch if New Sample>Last Sample
;
T5Interrupt_ChkIn8Max_Enabled_GT_Trigger:
		bclr	DirFlags,	#7			; Disable Peak Detection
		bset	TriggerFlags,	#7		; Trigger 8
		mov.b	WREG,		In8Max		; save the new sample
		mov		#P1,	W1				; Array Start
		mov		#3,		W0				; P1 Velocity offset
		add		W0,		W1,		W1
		mov.b	In8Max,	WREG
		mov.b	W0,		[W1]
		goto	T5Interrupt_ChkIn8Max_Exit	; all done here
;
T5Interrupt_ChkIn8Max_Enabled_GT_NotYet:
		goto	T5Interrupt_ChkIn8Max_Continue	; go save the new sample				
;
T5Interrupt_ChkIn8Max_Disabled:
		cpsne.b	W0,		W3				; skip next if New Sample<>Min_OFF			
		goto	T5Interrupt_ChkIn8Max_Exit
;
		cp.b	W0,		W3
		bra		GTU,	T5Interrupt_ChkIn8Max_Disabled_GT	; skip next if New Sample>Min_OFF			
;
T5Interrupt_ChkIn8Max_Disabled_LT:
		bset	DirFlags,	#7			; Enable Peak Detection
		goto	T5Interrupt_ChkIn7Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn8Max_Disabled_GT:
		goto	T5Interrupt_ChkIn8Max_Continue	; go save the new sample
;
T5Interrupt_ChkIn8Max_Continue:
		mov.b	WREG,	In8Max		; save the new sample		
;		
T5Interrupt_ChkIn8Max_Exit:	
;***************************************
T5Interrupt_Exit:
		bset	AD1CON1,	#ASAM		; do another sample
;        bclr	LATA,	#LED		 	; this is for testing only (scope)		
		pop.d	W2    
		pop.d	W0  		    		  
		retfie 		
;}
;***************************************;***************************************
;
;	Change notice Interrupt Service Routine
;
;	Requires:
;		none
;	Changes:
;		IFS1,	#CNIF
;		ThisPortA
;		ThisPortA
;		LastPortB
;		LastPortB
;		TriggerFlags,	#0
;		TriggerFlags,	#1
;		TriggerFlags,	#2
;		TriggerFlags,	#3
;		TriggerFlags,	#4
;		TriggerFlags,	#5
;		TriggerFlags,	#6
;		TriggerFlags,	#7
;		MyPad
; 	Calls:
;		none
;
;***************************************;***************************************
;{		
__CNInterrupt:
        push.d 	W0                  	; Save context using double-word PUSH
		mov		PORTA,	W0
		mov		PORTB,	W1
        bclr 	IFS1,	#CNIF         	; Clear the CN Interrupt flag bit.
;        		
		mov		W0,		ThisPortA
		mov		W1,		ThisPortB
;
CNInterrupt_ChkPortA:		
		mov		LastPortA,	W1
		xor		W0,		W1,		W1
		mov		W1,		LastPortA
;
CNInterrupt_ChkRA0:		
		btss	W1,		#RA0			; skip next if RA0 has changed
		goto	CNInterrupt_ChkRA1		; else go on
;
CNInterrupt_RA0:
		mov		#1,		W0				; Pad 1
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RA0		; skip next if RA0 is now ON
		goto	CNInterrupt_RA0_OFF
;
CNInterrupt_RA0_ON:
		bset	TriggerFlags,	#0		; Trigger 1		
		goto	CNInterrupt_ChkRA1
;
CNInterrupt_RA0_OFF:
		goto	CNInterrupt_ChkRA1	
;
CNInterrupt_ChkRA1:		
		btss	W1,		#RA1			; skip next if RA1 has changed
		goto	CNInterrupt_ChkRA2		; else go on
;
CNInterrupt_RA1:
		mov		#2,		W0				; Pad 2
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RA1		; skip next if RA1 is now ON
		goto	CNInterrupt_RA1_OFF
;
CNInterrupt_RA1_ON:
		bset	TriggerFlags,	#1		; Trigger 2	
		goto	CNInterrupt_ChkRA2
;
CNInterrupt_RA1_OFF:
		goto	CNInterrupt_ChkRA2
;		
CNInterrupt_ChkRA2:		
		btss	W1,		#RA2			; skip next if RA2 has changed
		goto	CNInterrupt_ChkRA3		; else go on
;
CNInterrupt_RA2:
		mov		#3,		W0				; Pad 3
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RB2		; skip next if RA2 is now ON
		goto	CNInterrupt_RA2_OFF
;
CNInterrupt_RA2_ON:
		bset	TriggerFlags,	#2		; Trigger 3
		goto	CNInterrupt_ChkRA3
;
CNInterrupt_RA2_OFF:
		goto	CNInterrupt_ChkRA3
;		
CNInterrupt_ChkRA3:		
		btss	W1,		#RA3			; skip next if RA3 has changed
		goto	CNInterrupt_ChkPortB	; else go on
;
CNInterrupt_RA3:
		mov		#4,		W0				; Pad 4
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RA3		; skip next if RA3 is now ON
		goto	CNInterrupt_RA3_OFF
;
CNInterrupt_RA3_ON:
		bset	TriggerFlags,	#3		; Trigger 4		
		goto	CNInterrupt_ChkPortB
;
CNInterrupt_RA3_OFF:
		goto	CNInterrupt_ChkPortB
;		
CNInterrupt_ChkPortB:
		mov		ThisPortB,	W0
		mov		LastPortB,	W1
		xor		W0,		W1,		W1
		mov		W1,		LastPortB
;
CNInterrupt_ChkRB12:		
		btss	W1,		#RB12			; skip next if RB12 has changed
		goto	CNInterrupt_ChkRB13		; else go on
;
CNInterrupt_RB12:
		mov		#5,		W0				; Pad 5
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RB12		; skip next if RB12 is now ON
		goto	CNInterrupt_RB12_OFF
;
CNInterrupt_RB12_ON:
		bset	TriggerFlags,	#4		; Trigger 5
		goto	CNInterrupt_ChkRB13
;
CNInterrupt_RB12_OFF:
		goto	CNInterrupt_ChkRB13
;
CNInterrupt_ChkRB13:		
		btss	W1,		#RB13			; skip next if RB13 has changed
		goto	CNInterrupt_ChkRB14		; else go on
;
CNInterrupt_RB13:
		mov		#6,		W0				; Pad 6
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RB13		; skip next if RB13 is now ON
		goto	CNInterrupt_RB13_OFF
;
CNInterrupt_RB13_ON:
		bset	TriggerFlags,	#5		; Trigger 6
		goto	CNInterrupt_ChkRB14
;
CNInterrupt_RB13_OFF:
		goto	CNInterrupt_ChkRB14
;		
CNInterrupt_ChkRB14:		
		btss	W1,		#RB14			; skip next if RB14 has changed
		goto	CNInterrupt_ChkRB15		; else go on
;
CNInterrupt_RB14:
		mov		#7,		W0				; Pad 7
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RB14		; skip next if RB14 is now ON
		goto	CNInterrupt_RB14_OFF
;
CNInterrupt_RB14_ON:
		bset	TriggerFlags,	#6		; Trigger 7
		goto	CNInterrupt_ChkRB15
;
CNInterrupt_RB14_OFF:
		goto	CNInterrupt_ChkRB15
;
CNInterrupt_ChkRB15:		
		btss	W1,		#RB15			; skip next if RB15 has changed
		goto	CNInterrupt_Save	; else go on
;
CNInterrupt_RB15:
		mov		#8,		W0				; Pad 8
		mov.b	WREG,	MyPad
		btsc	ThisPortA,	#RB15		; skip next if RB15 is now ON
		goto	CNInterrupt_RB15_OFF
;
CNInterrupt_RB15_ON:
		bset	TriggerFlags,	#7		; Trigger 8
		goto	CNInterrupt_Save
;
CNInterrupt_RB15_OFF:
		goto	CNInterrupt_Save
;
CNInterrupt_Save:
		mov		ThisPortA,	W0
		mov		W0,		LastPortA
		mov		ThisPortB,	W0
		mov		W0,		LastPortB								
;
CNInterrupt_Exit:
		pop.d	W0 		
		retfie
;}
;***************************************;***************************************
;
;	A/D Interrupt Service Routine	-	all channels have been read and stored
;										so just stop further samples
;
;	Requires:
;		none
;	Changes:
;		IFS0,	#AD1IF
;		AD1CON1,	#ASAM
; 	Calls:
;		none
;
;***************************************;***************************************
;{			
__ADC1Interrupt:
		bclr	IFS0,	#AD1IF			; clear the interrupt
		bclr	AD1CON1,	#ASAM
		retfie		
;}
;***************************************;***************************************
;
;	Tables	-	null terminated ROM strings (Messages)
;
;***************************************;***************************************
;{        
SignOn1:
		.asciz	"from the bench #274"
SignOn2:		
		.asciz	"  jeff bachiochi   "
SignOn3:
		.asciz	"imaginethatnow.com "
SignOn4:
		.asciz	"  Midi Monitor     "
SignOn5:
		.asciz	"  Midi Drum Kit    "
SignOn6:
		.asciz	"from the bench #275"						
Message:
		.asciz	"Message:"		
Message8:
		.asciz	"Note Off ("
Message9:
		.asciz	"Note On ("
MessageA:
		.asciz	"AfterTouch ("
MessageB:
		.asciz	"Control Change ("
MessageC:
		.asciz	"Program Change ("
MessageD:
		.asciz	"Channel Pressure ("												
MessageE:
		.asciz	"Pitch Wheel ("		
MessageF0:
		.asciz	"Start of SysEx ("	
MessageF1:
		.asciz	"MTC Quarter Frame, "
TimeCode:
		.asciz	"TimeCode:"		
MessageF2:
		.asciz	"Song Position Pointer, "
MidiBeat:
		.asciz	"MidiBeat:"			
MessageF3:
		.asciz	"Song Select, "
Song:
		.asciz	"Song:"	
MessageF6:
		.asciz	"Tune Request"	
MessageF7:
		.asciz	") End of SysEx"
MessageF8:
		.asciz	"Midi Clock"	
MessageF9:
		.asciz	"Tick"
MessageFA:
		.asciz	"Midi Start"
MessageFB:
		.asciz	"Midi Continue"	
MessageFC:
		.asciz	"Midi Stop"	
MessageFE:
		.asciz	"Active Sense"	
MessageFF:
		.asciz	"Reset"										
Channel:
		.asciz	"Channel:"
Note:
		.asciz	"Note:"
Velocity:
		.asciz	"Vel:"	
Pressure:
		.asciz	"Pressure:"
Controller:
		.asciz	"Controller:"
Value:
		.asciz	"Value:"
Program:
		.asciz	"Program:"
Transpose:
		.asciz	"Transpose:"
Pad:
		.asciz	"Pad:"
Instrument:																					                    
		.asciz	"Inst:"
Beats:																					                    
		.asciz	"BPM:"		
EntryError:
		.asciz 	"Entry Error"
Triggering:
		.asciz 	"Triggering... "
SavedSettings:
		.asciz 	"   Settings Saved   "
;}
;***************************************;***************************************
;
;	End
;
;***************************************;***************************************
;

.end                               ;End of program code in this file



